// $Id: RandBreitWigner.cpp,v 1.1.1.1 2000/04/03 21:03:45 wilko Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                       --- RandBreitWigner ---
//                      class implementation file
// -----------------------------------------------------------------------
// This file is part of Geant4 (simulation toolkit for HEP).

// =======================================================================
// Gabriele Cosmo - Created: 5th September 1995
//                - Added methods to shoot arrays: 28th July 1997
// J.Marraffino   - Added default arguments as attributes and
//                  operator() with arguments: 16th Feb 1998
// =======================================================================

#include "CLHEP/config/CLHEP.h"

#include "CLHEP/config/TemplateFunctions.h"
#include "CLHEP/Random/RandBreitWigner.h"
//#include <algorithm>

RandBreitWigner::~RandBreitWigner() {
  if ( deleteEngine ) delete localEngine;
}

RandBreitWigner::RandBreitWigner(const RandBreitWigner& right)
 : defaultA(right.defaultA), defaultB(right.defaultB)
{;}

HepDouble RandBreitWigner::operator()() {
   return fire( defaultA, defaultB );
}

HepDouble RandBreitWigner::operator()( HepDouble a, HepDouble b ) {
   return fire( a, b );
}

HepDouble RandBreitWigner::operator()( HepDouble a, HepDouble b, HepDouble c ) {
   return fire( a, b, c );
}

HepDouble RandBreitWigner::shoot(HepDouble mean, HepDouble gamma)
{
   HepDouble rval, displ;

   rval = 2.0*HepRandom::getTheEngine()->flat()-1.0;
   displ = 0.5*gamma*tan(rval*M_PI_2);

   return mean + displ;
}

HepDouble RandBreitWigner::shoot(HepDouble mean, HepDouble gamma, HepDouble cut)
{
   HepDouble val, rval, displ;

   if ( gamma == 0.0 ) return mean;
   val = atan(2.0*cut/gamma);
   rval = 2.0*HepRandom::getTheEngine()->flat()-1.0;
   displ = 0.5*gamma*tan(rval*val);

   return mean + displ;
}

HepDouble RandBreitWigner::shootM2(HepDouble mean, HepDouble gamma )
{
   HepDouble val, rval, displ;

   if ( gamma == 0.0 ) return mean;
   val = atan(-mean/gamma);
   rval = RandFlat::shoot(val, M_PI_2);
   displ = gamma*tan(rval);

   return sqrt(mean*mean + mean*displ);
}

HepDouble RandBreitWigner::shootM2(HepDouble mean, HepDouble gamma, HepDouble cut )
{
   HepDouble rval, displ;
   HepDouble lower, upper, tmp;

   if ( gamma == 0.0 ) return mean;
   tmp = std::max(0.0,(mean-cut));
   lower = atan( (tmp*tmp-mean*mean)/(mean*gamma) );
   upper = atan( ((mean+cut)*(mean+cut)-mean*mean)/(mean*gamma) );
   rval = RandFlat::shoot(lower, upper);
   displ = gamma*tan(rval);

   return sqrt(std::max(0.0, mean*mean + mean*displ));
}

void RandBreitWigner::shootArray ( const HepInt size, HepDouble* vect )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot( 1.0, 0.2 );
}

void RandBreitWigner::shootArray ( const HepInt size, HepDouble* vect,
                                   HepDouble a, HepDouble b )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot( a, b );
}

void RandBreitWigner::shootArray ( const HepInt size, HepDouble* vect,
                                   HepDouble a, HepDouble b,
                                   HepDouble c )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot( a, b, c );
}

//----------------

HepDouble RandBreitWigner::shoot(HepRandomEngine* anEngine,
                                 HepDouble mean, HepDouble gamma)
{
   HepDouble rval, displ;

   rval = 2.0*anEngine->flat()-1.0;
   displ = 0.5*gamma*tan(rval*M_PI_2);

   return mean + displ;
}

HepDouble RandBreitWigner::shoot(HepRandomEngine* anEngine,
                                 HepDouble mean, HepDouble gamma, HepDouble cut )
{
   HepDouble val, rval, displ;

   if ( gamma == 0.0 ) return mean;
   val = atan(2.0*cut/gamma);
   rval = 2.0*anEngine->flat()-1.0;
   displ = 0.5*gamma*tan(rval*val);

   return mean + displ;
}

HepDouble RandBreitWigner::shootM2(HepRandomEngine* anEngine,
                                   HepDouble mean, HepDouble gamma )
{
   HepDouble val, rval, displ;

   if ( gamma == 0.0 ) return mean;
   val = atan(-mean/gamma);
   rval = RandFlat::shoot(anEngine,val, M_PI_2);
   displ = gamma*tan(rval);

   return sqrt(mean*mean + mean*displ);
}

HepDouble RandBreitWigner::shootM2(HepRandomEngine* anEngine,
                                   HepDouble mean, HepDouble gamma, HepDouble cut )
{
   HepDouble rval, displ;
   HepDouble lower, upper, tmp;

   if ( gamma == 0.0 ) return mean;
   tmp = std::max(0.0,(mean-cut));
   lower = atan( (tmp*tmp-mean*mean)/(mean*gamma) );
   upper = atan( ((mean+cut)*(mean+cut)-mean*mean)/(mean*gamma) );
   rval = RandFlat::shoot(anEngine, lower, upper);
   displ = gamma*tan(rval);

   return sqrt( std::max(0.0, mean*mean+mean*displ) );
}

void RandBreitWigner::shootArray ( HepRandomEngine* anEngine,
                                   const HepInt size, HepDouble* vect )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot( anEngine, 1.0, 0.2 );
}

void RandBreitWigner::shootArray ( HepRandomEngine* anEngine,
                                   const HepInt size, HepDouble* vect,
                                   HepDouble a, HepDouble b )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot( anEngine, a, b );
}

void RandBreitWigner::shootArray ( HepRandomEngine* anEngine,
                                   const HepInt size, HepDouble* vect,
                                   HepDouble a, HepDouble b, HepDouble c )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot( anEngine, a, b, c );
}

//----------------

HepDouble RandBreitWigner::fire()
{
  return fire( defaultA, defaultB );
}

HepDouble RandBreitWigner::fire(HepDouble mean, HepDouble gamma)
{
   HepDouble rval, displ;

   rval = 2.0*localEngine->flat()-1.0;
   displ = 0.5*gamma*tan(rval*M_PI_2);

   return mean + displ;
}

HepDouble RandBreitWigner::fire(HepDouble mean, HepDouble gamma, HepDouble cut)
{
   HepDouble val, rval, displ;

   if ( gamma == 0.0 ) return mean;
   val = atan(2.0*cut/gamma);
   rval = 2.0*localEngine->flat()-1.0;
   displ = 0.5*gamma*tan(rval*val);

   return mean + displ;
}

HepDouble RandBreitWigner::fireM2()
{
  return fireM2( defaultA, defaultB );
}

HepDouble RandBreitWigner::fireM2(HepDouble mean, HepDouble gamma )
{
   HepDouble val, rval, displ;

   if ( gamma == 0.0 ) return mean;
   val = atan(-mean/gamma);
   rval = RandFlat::shoot(localEngine,val, M_PI_2);
   displ = gamma*tan(rval);

   return sqrt(mean*mean + mean*displ);
}

HepDouble RandBreitWigner::fireM2(HepDouble mean, HepDouble gamma, HepDouble cut )
{
   HepDouble rval, displ;
   HepDouble lower, upper, tmp;

   if ( gamma == 0.0 ) return mean;
   tmp = std::max(0.0,(mean-cut));
   lower = atan( (tmp*tmp-mean*mean)/(mean*gamma) );
   upper = atan( ((mean+cut)*(mean+cut)-mean*mean)/(mean*gamma) );
   rval = RandFlat::shoot(localEngine,lower, upper);
   displ = gamma*tan(rval);

   return sqrt(std::max(0.0, mean*mean + mean*displ));
}


void RandBreitWigner::fireArray ( const HepInt size, HepDouble* vect)
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire(defaultA, defaultB );
}

void RandBreitWigner::fireArray ( const HepInt size, HepDouble* vect,
                                  HepDouble a, HepDouble b )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire( a, b );
}

void RandBreitWigner::fireArray ( const HepInt size, HepDouble* vect,
                                  HepDouble a, HepDouble b, HepDouble c )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire( a, b, c );
}
