// $Id: RandFlat.cpp,v 1.1.1.1 2000/04/03 21:03:45 wilko Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                          --- RandFlat ---
//                      class implementation file
// -----------------------------------------------------------------------
// This file is part of Geant4 (simulation toolkit for HEP).

// =======================================================================
// Gabriele Cosmo - Created: 17th May 1995
//                - Added methods to shoot arrays: 28th July 1997
//                - Added operator(): 24th Jul 1997
// J.Marraffino   - Added default arguments as attributes and
//                  operator() with arguments: 16th Feb 1998
// =======================================================================

#include "CLHEP/Random/RandFlat.h"

const HepInt RandFlat::MSBBits= 15;
const unsigned long RandFlat::MSB= 1ul<<RandFlat::MSBBits;
unsigned long RandFlat::staticRandomInt= 0;
unsigned long RandFlat::staticFirstUnusedBit= 0;

RandFlat::~RandFlat() {
  if ( deleteEngine ) delete localEngine;
}

RandFlat::RandFlat(const RandFlat& right)
 : defaultWidth(right.defaultWidth), defaultA(right.defaultA),
   defaultB(right.defaultB)
{;}

HepDouble RandFlat::operator()() {
  return fire( defaultA, defaultB );
}

HepDouble RandFlat::operator()( HepDouble w ) {
  return fire( w );
}

HepDouble RandFlat::operator()( HepDouble a, HepDouble b ) {
  return fire( a, b );
}

HepDouble RandFlat::shoot() {
  return HepRandom::getTheEngine()->flat();
}

void RandFlat::shootArray(const HepInt size, HepDouble* vect) {
  HepRandom::getTheEngine()->flatArray(size,vect);
}

void RandFlat::shootArray( const HepInt size, HepDouble* vect,
                           HepDouble lx, HepDouble dx  )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot(lx,dx);
}

void RandFlat::shootArray( HepRandomEngine* anEngine,
                           const HepInt size, HepDouble* vect,
                           HepDouble lx, HepDouble dx  )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot(anEngine,lx,dx);
}

void RandFlat::fireArray( const HepInt size, HepDouble* vect)
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire( defaultA, defaultB );
}

void RandFlat::fireArray( const HepInt size, HepDouble* vect,
                          HepDouble lx, HepDouble dx  )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire( lx, dx );
}
