// $Id: RandGauss.cpp,v 1.1.1.1 2000/04/03 21:03:45 wilko Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                          --- RandGauss ---
//                      class implementation file
// -----------------------------------------------------------------------
// This file is part of Geant4 (simulation toolkit for HEP).

// =======================================================================
// Gabriele Cosmo - Created: 5th September 1995
//                - Added methods to shoot arrays: 28th July 1997
// J.Marraffino   - Added default arguments as attributes and
//                  operator() with arguments. Introduced method normal()
//                  for computation in fire(): 16th Feb 1998
// Gabriele Cosmo - Relocated static data from HepRandom: 5th Jan 1999
// =======================================================================

#include "CLHEP/Random/RandGauss.h"

// Initialisation of static data
HepBoolean RandGauss::set_st = false;
HepDouble RandGauss::nextGauss_st = 0.0;

RandGauss::~RandGauss() {
  if ( deleteEngine ) delete localEngine;
}

RandGauss::RandGauss(const RandGauss& right)
 : defaultMean(right.defaultMean), defaultStdDev(right.defaultStdDev)
{;}

HepDouble RandGauss::operator()() {
  return fire( defaultMean, defaultStdDev );
}

HepDouble RandGauss::operator()( HepDouble mean, HepDouble stdDev ) {
  return fire( mean, stdDev );
}

HepDouble RandGauss::shoot()
{
  // Gaussian random numbers are generated two at the time, so every other
  // time this is called we just return a number generated the time before.

  if ( getFlag() ) {
    setFlag(false);
    return getVal();
  }

  HepDouble r;
  HepDouble v1,v2,fac,val;
  HepRandomEngine* anEngine = HepRandom::getTheEngine();

  do {
    v1 = 2.0 * anEngine->flat() - 1.0;
    v2 = 2.0 * anEngine->flat() - 1.0;
    r = v1*v1 + v2*v2;
  } while ( r > 1.0 );

  fac = sqrt(-2.0*log(r)/r);
  val = v1*fac;
  setVal(val);
  setFlag(true);
  return v2*fac;
}

void RandGauss::shootArray( const HepInt size, HepDouble* vect,
                            HepDouble mean, HepDouble stdDev )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot(mean,stdDev);
}

HepDouble RandGauss::shoot( HepRandomEngine* anEngine )
{
  // Gaussian random numbers are generated two at the time, so every other
  // time this is called we just return a number generated the time before.

  if ( getFlag() ) {
    setFlag(false);
    return getVal();
  }

  HepDouble r;
  HepDouble v1,v2,fac,val;

  do {
    v1 = 2.0 * anEngine->flat() - 1.0;
    v2 = 2.0 * anEngine->flat() - 1.0;
    r = v1*v1 + v2*v2;
  } while ( r > 1.0 );

  fac = sqrt( -2.0*log(r)/r);
  val = v1*fac;
  setVal(val);
  setFlag(true);
  return v2*fac;
}

void RandGauss::shootArray( HepRandomEngine* anEngine,
                            const HepInt size, HepDouble* vect,
                            HepDouble mean, HepDouble stdDev )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot(anEngine,mean,stdDev);
}

HepDouble RandGauss::normal()
{
  // Gaussian random numbers are generated two at the time, so every other
  // time this is called we just return a number generated the time before.

  if ( set ) {
    set = false;
    return nextGauss;
  }

  HepDouble r;
  HepDouble v1,v2,fac,val;

  do {
    v1 = 2.0 * localEngine->flat() - 1.0;
    v2 = 2.0 * localEngine->flat() - 1.0;
    r = v1*v1 + v2*v2;
  } while ( r > 1.0 );

  fac = sqrt(-2.0*log(r)/r);
  val = v1*fac;
  nextGauss = val;
  set = true;
  return v2*fac;
}

void RandGauss::fireArray( const HepInt size, HepDouble* vect)
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire( defaultMean, defaultStdDev );
}

void RandGauss::fireArray( const HepInt size, HepDouble* vect,
                           HepDouble mean, HepDouble stdDev )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire( mean, stdDev );
}
