// -*- C++ -*-
// $Id: LorentzVector.cpp,v 1.1.1.1 2000/04/03 21:03:47 wilko Exp $
// ---------------------------------------------------------------------------
//
// This file is a part of the CLHEP - a Class Library for High Energy Physics.
//
// This is the implementation of the HepLorentzVector class.
//

#ifdef __GNUG__
#pragma implementation
#endif

#include "CLHEP/config/CLHEP.h"
#include "CLHEP/Vector/LorentzVector.h"
#include "CLHEP/Vector/LorentzRotation.h"

// #include "CLHEP/String/Strings.h"
#include "CLHEP/config/strstream.h"
#include <ctype.h>

#ifdef HEP_DEBUG_INLINE
#include "CLHEP/Vector/LorentzVector.inl"
#endif

HepDouble HepLorentzVector::operator () (int i) const {
  switch(i) {
  case X:
  case Y:
  case Z:
    return pp(i);
  case T:
    return e();
  default:
    std::cerr << "HepLorentzVector subscripting: bad index (" << i << ")"
	      << std::endl;
  }
  return 0.;
}  

HepDouble & HepLorentzVector::operator () (int i) {
  switch(i) {
  case X:
  case Y:
  case Z:
    return pp(i);
  case T:
    return ee;
  default:
    std::cerr << "HepLorentzVector subscripting: bad index (" << i << ")"
	      << std::endl;
  }
  HepDouble dummy;
  HepDouble & rdummy = dummy;
  return rdummy;
}

void HepLorentzVector::boost(HepDouble bx, HepDouble by, HepDouble bz){
  HepDouble b2 = bx*bx + by*by + bz*bz;
  register HepDouble gamma = 1.0 / sqrt(1.0 - b2);
  register HepDouble bp = bx*x() + by*y() + bz*z();
  register HepDouble gamma2 = b2 > 0 ? (gamma - 1.0)/b2 : 0.0;

  setX(x() + gamma2*bp*bx + gamma*bx*t());
  setY(y() + gamma2*bp*by + gamma*by*t());
  setZ(z() + gamma2*bp*bz + gamma*bz*t());
  setT(gamma*(t() + bp));
}

HepDouble HepLorentzVector::rapidity() const {
  return 0.5*log( (e()+pz()) / (e()-pz()) );
}

HepLorentzVector &
HepLorentzVector::operator *= (const HepLorentzRotation & m) {
  return *this = m.vectorMultiplication(*this);
}

HepLorentzVector &
HepLorentzVector::transform(const HepLorentzRotation & m){
  return *this = m.vectorMultiplication(*this);
}

std::ostream & operator << (std::ostream & s, const HepLorentzVector & q)
{
  return s << "(" << q.x() << "," << q.y() << ","
           << q.z() << ";" << q.t() << ")";
}

#if 0
std::istream & operator >> (std::istream & s, HepLorentzVector & q) {
  // Parse:
  HepString cleanString;
  char c;
  int numRead = 0;
  HepBoolean previosIsSpace = false;

  while(EOF != (c = s.get()) && c != ')' && s.good()) {
    // ignore initial bracket or spaces
    if (!cleanString.isEmpty() || ('(' != c && 0 == isspace(c))){            
      if (',' == c){          // replace ',' with ' '
	c = ' ';
      }
      else if (';' == c){          // replace ';' with ' '
	c = ' ';
      }
      cleanString += c;     // take c
      if (isspace(c)){      // count # of white spaces (excluding initials)
	if (false == previosIsSpace) {
	  ++numRead;
	}
	if (numRead >= q.SIZE){  // read enough
	  break;
	}
      }
      else {
	previosIsSpace = false;
      }
    }
  }

  std::istrstream newStream(cleanString);
  double xx(q.x()), yy(q.y()), zz(q.z()), tt(q.t());
  newStream >> xx >> yy >> zz >> tt;
  q.setX(xx);
  q.setY(yy);
  q.setZ(zz);
  q.setT(tt);

  return s;
}

#endif
