#include "Event/IOfileServer.h"

#include "Event/messageManager.h"
#include "Event/optionManager.h"

//###############################
IOfileServer::IOfileServer()
//###############################
{
	m_ifile = -1;
	m_InFileList.clear();
	m_InFile  =  "";
	m_histoFile   ="";
	m_OutFile ="";
	m_displayFreq = 100;

	m_input = 0;
	m_output = 0;
	m_histo = 0;

	m_server = new serverVI<IOfileVI>();

	/*! call defineIOfiles, the class where the user should implement the input,
	output and histo file classes, using the \c typedef */ 
	defineIOfiles();

	setName("IOfileServer");
}
//###############################
IOfileServer::~IOfileServer()
//###############################
{
}
//#######################################
IOfileVI* IOfileServer::getIOfile(std::string name) const
//#######################################
{
	if (m_server->search(name)) return m_server->get(name);
	else messageManager::instance()->message("Not found IOfileV "+name);
	return 0;
}
//#######################################
void IOfileServer::open()
//#######################################
{
	/*! open the input file */
	std::string name = getNextInFileName();
	if (name !="") {
		getIOfile("input")->setFileName(name);
		getIOfile("input")->setMode(IOBase::READ);
		getIOfile("input")->open();
		if (getIOfile("input")->isOpen()) {
			message(" opened input file "+name);
			m_input = getIOfile("input");
		} else message(" could not open input file "+name);
		setNumEvents(getIOfile("input")->numEvents());
	}
	
	/*! open the histogram file */
	name = getHistoFileName();
	if (name !="") {
		getIOfile("histo")->setFileName(name);
		getIOfile("histo")->setMode(IOBase::WRITE);
		getIOfile("histo")->open();
		if (getIOfile("histo")->isOpen()) {
			message(" opened histo file "+name);
			m_histo = getIOfile("histo");
		} else message(" could not open histo file "+name);
	}
		
	/*! open the output file */
	name = getOutFileName();
	if (name !="") {
		getIOfile("output")->setFileName(name);
		getIOfile("output")->setMode(IOBase::WRITE);
		getIOfile("output")->open();
		if (getIOfile("output")->isOpen()) {
			message(" opened output file "+name);
			m_output = getIOfile("output");
		} else message(" could not open output file "+name);
	}
}
//#######################################
bool IOfileServer::nextEvent()
//#######################################
{
	if (search("input"))  return getIOfile("input")->nextEvent();
	return false;
}
//#######################################
void IOfileServer::readEvent()
//#######################################
{
	increase();
	if (displayFrequency()) message(" reading event ",eventNumber(),"GENERAL");;
	if (m_input)  m_input->readEvent();
}
//#######################################
void IOfileServer::writeEvent()
//#######################################
{
	if (m_output) m_output->writeEvent();
}

//#######################################
void IOfileServer::skipEvent()
//#######################################
{
	increase();
	if (displayFrequency()) message(" skiping event ",eventNumber(),"GENERAL");
	if (m_input)  m_input->skipEvent();
}
//#######################################
void IOfileServer::close()
//#######################################
{
	if (search("input"))  getIOfile("input")->close();
	if (search("output")) getIOfile("output")->close();
	if (search("histo"))  getIOfile("histo")->close();
}

//##################################
bool IOfileServer::displayFrequency() const
//##################################
{
	return (eventNumber()%m_displayFreq == 0); 
}
//##################################
std::string IOfileServer::getNextInFileName()
//##################################
{
	m_ifile++;
	std::string empty = "";
	if (m_ifile >= m_InFileList.size()) return empty;
	else return m_InFileList[m_ifile];
}
//########################################
void IOfileServer::defineOption()
//########################################
{
	optionVI::defineOption("InFile",&m_InFile);
	optionVI::defineOption("histoFile",&m_histoFile);
	optionVI::defineOption("OutFile",&m_OutFile);

	optionVI::defineOption("displayFrequency",&m_displayFreq);
}
//###############################
void IOfileServer::setOption(std::string par, std::string v)
//###############################
{
	optionVI::setOption(par,v);
	/*! if the file is "InFile", it added into the list of input files */
	if (par == "InFile") m_InFileList.push_back(v);
}
//###############################
void IOfileServer::writeOut() const
//###############################
{
	if (!acceptLevel()) return;
	messageVI::writeOut();
	/*! write Out the names of the input, output and histograms files */
	std::ostream& out = messageManager::instance()->out();
	out << " input  file name    = " << m_InFile <<"\n";
	out << " output file name    = " << m_OutFile <<"\n";
	out << " histos file name    = " << m_histoFile <<"\n";

}
