#ifndef evtProcess_h
#define evtProcess_h

#include "Event/eventComposite.h"
#include "Event/serviceI.h"
//----------------------------------------------
//! and eventProcess is a eventComposite with serviceI services
/*!
\e eventProcess is an algorithmVI.<br>
\e eventProcess is an eventVI. <br>
\e eventProcess is an eventComposite. <br>
\e Users can add dinamically: a) and eventVI to the composite b) algorithms
to the generation, reconstruction, analysis tasks of the eventVI.
<ul>
	<li> Is can use all the services provided by the base class serviceI. 
	To be included in the server every object of eventComposite 
	should have a unique name (use \c setName() method of serviceI to get services for
	an object).
	<li> Via the \c setOption() method of optionVI, and external user (optionManager) can:
	<ul>
		<li> add an eventComposite to this eventProcess (also eventComposite).
		<li> add algorithms to the algorithmComposite
		of the eventVI (main eventVI of the eventComposite) in the 
		generation, reconstruction, and analysis taks of this eventComposite.
	</ul>
</ul>
*/
//----------------------------------------------
//             J.A Hernando, Santa Cruz 02/29/00
//----------------------------------------------

//##################################
class eventProcess : public eventComposite, public serviceI
//##################################
{
public:

	//! constructor - set the name
	eventProcess(std::string name);
	//! default destructor
	~eventProcess();

	//! information of this class 
	virtual void writeOut() const;

protected:

	//! define the options (operations) into optionVI
	virtual void defineOption();
	//! It performs the operations associated with the options
	virtual void setOption(std::string task, std::string value); // overwriten

private:

	//! dummy strings for the option
	/*! The have the last algorithm name added in this object*/
	std::string m_genName;
	std::string m_recName;
	std::string m_anaName;

	//! the last eventProcess name added to this eventProcess
	std::string m_eventProcessName;
};
#endif
