#ifndef IOfileVI_h
#define IOfileVI_h

#include "Event/IOBase.h"

#include <string>

//------------------------------------------------
//! virtual inteface to handle a file with persistency data
/*!
<ul>
	<li> It handles the \c open(), \c close() of files. It knows if the
	file are type MODE READ or WRITE (see enumeration in IOBase).
	<li> It handles the \c readEvent(), \c writeEvent() and \c skipEvent() processes
	<li> It keeps contability of the number of events in the file (\c numEvents()), and number
	of event read or writen (\c eventNumber()).
</ul>
*/
//-------------------------------------------------
// JA Hernando, Santa Cruz, CA, 06/15/00
//-------------------------------------------------

//###########################
class IOfileVI
//###########################
{
public:

	//! default constructor
	IOfileVI(){clear();}
	//! default destructor
	~IOfileVI(){}

	//! set the name of the file
	virtual void setFileName(std::string name) {m_nameFile = name;}
	//! set the mode 
	virtual void setMode(IOBase::MODE m)       {m_mode = m;}
	//! set total number of events
	virtual void setNumEvents(int nevt)        {m_numEvents = nevt;}
	//! set the file to Open
	virtual void setOpen(bool t)               {m_open = t;}
	//! virtual method to increase one event
	virtual void increase()                    {m_currentEvent++;}

	//! virtual method to open the file
	virtual void open() = 0;
	//! virtual method to close the file
	virtual void close() = 0;

	//! virtual method to know if there is one event more in the file
	virtual bool nextEvent() { return (m_currentEvent+1 <= m_numEvents);}
	//! virtual method to read the next event - overwrite by the derived class
	virtual void readEvent()  {increase();}
	//! virtual method to write an event - overwrite by the derived class
	virtual void writeEvent() {increase();m_numEvents++;}
	//! virtual method to skip next event - overwrite by the derived class
	virtual void skipEvent()  {increase();}

	//! returns the name of the file
	virtual std::string getFileName()  const {return m_nameFile;}
	//! returns the mode READ or WRITE
	IOBase::MODE getMode()             const {return m_mode;}
	//! virtual method to return the current Event
	virtual int eventNumber()         const {return m_currentEvent;}
	//! virtual method ro return the total number of events;
	virtual int numEvents()            const {return m_numEvents;}
	//! virtual method to know if the file is open or not
	virtual bool isOpen()              const {return m_open;}

protected:

	//! clear all the variables
	virtual void clear();

private:

	//! name of the file
	std::string m_nameFile;
	//! mode of open the file READ or WRITE
	IOBase::MODE m_mode;
	//! is the file open?
	bool m_open;

	//! number of total events in the file
	int m_numEvents;
	//! the current event number
	int m_currentEvent;
};
#endif