#ifndef ALGORITHMVI_H
#define ALGORITHMVI_H


#include <string>

//-----------------------------------------------
//! Base class for algorithms
/*! 
\c algorithms are "executable" classes that runs with the method \c run().<br>
\c algorithms should inherit from algorithmVI.<br>
<ul>
	<li> the main virtual method \c run() executes the following methods: \c initialize(), 
	\c execute() and \c finalize().
	<li> the methods \c initialize(), \c execute(), \c and \c finalize() are pure virtual.
	They have to be implemented in the derived classes.
	<li> the method \c command() allows to execute the following name commands: "run",
	"initialize","execute","finalize", each command executes the method of its
	name. This is useful dealing with composites, in particular it is used by
	the serverVI template class.
</ul>
*/
//----------------------------------------------
//             J.A Hernando, Santa Cruz 02/29/00
//----------------------------------------------
//###########################
class algorithmVI 
//###########################
{
public:

	//! default constructor
	algorithmVI(){}
	//! default destructor
	~algorithmVI() {}

	//! virtual method to run the algorithm 
	/*! executes the methods: initialize(); execute(); finalize(); */
	virtual void run();

	//! pure virtual method initialize the algorithm
	virtual void initialize() = 0;
	//! pure virtual method to execute the algorithm
	virtual void execute()    = 0;
	//! pure virtual method to finalize the algorithm
	virtual void finalize()   = 0;

	//! virtual method to execute a command
	/*! The commands are the methods "run", "initialize", "execute", "finalize" */
	virtual void command(std::string com);

};
#endif
