#ifndef dataDetectorServer_H
#define dataDetectorServer_H

#include "Event/trsDataVI.h"
#include "Event/serverVI.h"
#include "Event/serviceI.h"

#include "Event/serverVI.h"

#include "Event/userDetectorGeo.h"

//----------------------------------------------
//! stores, servers, updates the detector data (geometry and calibration data).
/*!
\e dataDetectorServer has two type of data: geometry and calibration (also called transient RUN data).<br>
\e geometry data contains the detector geometry. There is no a centella solution
to the geometry description. The user should provide in userDetectorGeo a typedef to
his/her own class detector (maybe to a GEANT4 mother class).<br>
\e calibration data is declared to be transient data. The user should
implement the calibration data classes, they should inherit from trsDataVI. 
The user should create those classes in the class userDataDetector 
and add them into the this server. 
<ul>
	<li> dataDetectorServer has accesible pointers to the detector geometry data.
	<li> dataDetectorServer has a protected server with the detector calibration data.
	<ul>
		<li> The user can add transient calibration data using the \c addData() method.
		The calibration data is declared transiend data (inherits from trsDataVI).
		<li> The calibration data is accesible from dataManager instead of from this
		class, via the method \c getData(). 
	<li> Any transient calibration data can be updated executing the \c update() method. 
	</ul>
</ul>
*/
//----------------------------------------------
// J.A Hernando, Santa Cruz, CA 02/29/00
//----------------------------------------------
//###############################
class dataDetectorServer : public serviceI
//###############################
{
	friend class dataManager;
	friend class comLoadCalibration;

public:

	//! construct the data
	dataDetectorServer();
	//! delete the data
	~dataDetectorServer();

	//! returns the geometry
	virtual detectorGeo* geo() {return m_detectorGeo;}

	//! add transient data into the server
	virtual void addData(std::string name, trsDataVI* data);

	//! virtual void update - update all classes in the server
	virtual void update() {m_server->command("update");}

	//! update the transient data wiht name
	virtual void update(std::string name);

	//! write out the information of all the data classes
	virtual void writeOutServer() const {m_server->command("writeOut");}
	//! write out information of a given detector data class
	virtual void writeOut(std::string name) const {m_server->get(name)->writeOut();}
	//! write out the information of the data
	virtual void writeOut() const;

protected:

	//! returns the trsDataVI with the name for geometry and calibration data
	virtual trsDataVI* getTrsData(std::string name) const; 

private:

	//! pointer to the detector geometry
	detectorGeo* m_detectorGeo;

	//! pointer to the server of the trsDataVI of the transient data for the detector
	serverVI<trsDataVI>* m_server;

};
#endif
