#ifndef defineVI_h
#define defineVI_h
//-------------------------------------
//! Bare clas to allow classes to be defined at run-time 
/*! 
<ul>
	<li> A derived class of defineVI can check at run-time if it has been already defined or not.
	If not, it can execute the \c define() method and define itself. 
	<li> Lets consider the example of multialgorithms, that used other algorithms.
	At compile time maybe some algorithms are not beed defined yet. The class
	is therefore not defined at compile time. At run time, when the
	\c initialize() (of algorithmVI) method is executed the first time,
	the derived class will them execute its \c define() method and define its algorithms. 
	Setting the \c setDefine() to true, the define() method will not be executed again. 
	<li> This class is useful to retreive at one, pointers that will keep their address
	constant during the live of the class.
</ul>
*/
//-------------------------------------
// JA Hernando, Santa Cruz, CA, 05/15/00
//-------------------------------------
//#####################################
class defineVI
//#####################################
{
protected:

	//! constructor - set definition to false
	defineVI() {setDefine(false);}
	//! destructor
	~defineVI() {};

	//! pure virtual class to define a class
	virtual void define() = 0;

	//! set the definition
	void setDefine(bool t)  {m_define = t;}
	//! returns false if the class has not bee defined
	bool defined() const {return m_define;}

private:

	//! keeps track if the clas has been run time defined or not
	bool m_define;
};
#endif
