#ifndef rHistoFolderServer_h
#define rHistoFolderServer_h

#include "Event/rHistoFolder.h"
#include "Event/serverVI.h"

#include <vector>
//--------------------------------------
//! stores, serves, fills a collection of rHistoFolder s
/*! 
<ul>
	<li> It has a server with rHistoFolder s. 
	<li> But the rHistoFolders are created and defined by a derived class that should implement the pure
	virtual method \c defineHistoFolders(). Every rHistoFolder should be incorporated
	in the server via the method \c addHistoFolder().
	<li> The rHistoFolders incorporated in ther server will turn to be algorithms
	via the method \c makeAlgorithms(). This method is automatically executed after
	\c defineHistoFolders(). The name of the algorithm will be fill+"name of the
	folder in this server" (i.e if the rHistoFolder is named \e "trackHisto", the algorithm
	will be named \e "filltrackHisto").
	<li> The \c fill() or \c define() of every rHistoFolder in the server can
	be executed by the name of the folder.
	<li> It provides protected access to a rHistofolder by its name (method \c getHistoFolder()).
</ul>
*/
//---------------------------------------
// JA Hernando, Santa Cruz, CA, 05/15/00
//---------------------------------------
//#############################
class rHistoFolderServer 
//#############################
{
public:	
	
	//! execute \c define() the rHistoFolder with name
	void define(std::string name) {getHistoFolder(name)->define();}
	//! execute \c fill()  in the rHistoFolder with name
	void fill(std::string name)   {getHistoFolder(name)->fill();}

protected:

	//! constructor.
	rHistoFolderServer();
	//! destructor
	~rHistoFolderServer();

	//! returns the rHistoFolder named with "name" 
	rHistoFolder* getHistoFolder(std::string name) const;


	//! virtual method. The user should define the folders
	/*! execute \c makeAlgorithms() to create the algorithms associated with the folders*/
	virtual void defineHistoFolders() {makeAlgorithms();};
	//! add rHistoFolder to the server
	virtual void addHistoFolder(std::string name, rHistoFolder* folder);

	//! converts the user defined rHistoFolders into algorithms
	virtual void makeAlgorithms();

	//! set the root-file where histos/ntuple are writen for all "rHistoFolder" in the Server
	virtual void setFile(TFile* file);

private:

	//! serverVI of the "rHistoFolder" in the Server
	serverVI<rHistoFolder>* m_server;
};
#endif