
#ifndef rTreeBase_h
#define rTreeBase_h

#include "rTFileBase.h"

#include "TROOT.h"

class TTree;
//-------------------------------------------
//! A template class with IOfileVI interface to use a ROOT-Tree
/*! 
\e rTreeBase has all the IOfileVI operations defined (open, close, readEvent, writeEvent, etc). <br>  
\e rTreeBase class needs the main branch class of the ROOT tree as a template argument.
The name of the class is also needed by ROOT. The template and its name are forced
arguments of the constructor!.
<ul>	
	<li> The Tree should have only one main branch, the main branch is defined 
	using the template "rCore". 
	<li> The operations of \c open() and \c close() are overwriten from rTFileBase,
	in order to set the address of the main branch of the tree to the template class rCore.
	<li> It provides the method to \c read(), \c write() from/to a ROOT-tree.<br>
	<li> the IOfileVI interface cames from rTFileBase.
</ul>
*/
//----------------------------------------
// JA Hernando, Santa Cruz, CA, 05/15/00
//----------------------------------------
//########################################
template<class rCore> class rTreeBase : public rTFileBase
//########################################
{	
public:

	//! constructor: it need the main branch class "rEvent" template
	rTreeBase(rCore* r, char* name):m_core(r),m_class(name)  {clear();};
	//! destructor
	~rTreeBase() {};  

	//! open a (read or write) root-tree file and (load or create) the tree
	/*! It set the branchAddess of the tree to the main branch class "rEvent" */
	virtual void open(); // init with this root file
	//! clear the member variables
	virtual void clear();

	/// returns the main branch class
	rCore* core()    {return m_core;}
    // returns the name of the top-level object in the tree
    char* className()       {return m_class; }

	//operations
	//! write current event from the main branch into the root-tree 
	void writeEvent();   
	//! read current event data from the root-tree into the main branch
	void readEvent();    
	//! skip some events when reading
	void skipEvent() {IOfileVI::skipEvent();}

protected:

	//! pointer to the root-tree
	TTree*        m_RTree; //pointer to the analyzed TTree

	//! main branch class of the tree (template to be instanciated by the user)
	rCore* m_core;

	//! root file buffer size
    UInt_t bufsize; 
	//! root file split level
    Int_t split;   

    char* m_class;   // name of the top-level class to retrieve 
};
#endif

