
#include "Event/runEventAlg.h"

#include "Event/selectionManager.h"
#include "Event/processManager.h"

//###################################
runEventAlg::runEventAlg()
//###################################
{
	m_selRead  = "selRead";
	m_selWrite = "selWrite";

	m_comReadEvent = "comReadEvent";
	m_comSkipEvent = "comSkipEvent";
	m_comWriteEvent = "comWriteEvent";

	m_runEvent      = "runEvent";

	setName("runEventAlg");
	defineOption();
}
//###################################
void runEventAlg::execute()
//###################################
{
	if (!defined()) define();

	//! should we read the next event
	if (m_cReadEvent->apply()) {
		//! read the event
		m_aReadEvent->run();
		//! process the event
		m_aProcessEvent->run();
		//! should we write this event, if yes, write it
		if (m_cWriteEvent->apply()) m_aWriteEvent->run();
	} else {
		m_aSkipEvent->run();
	}
}
//###################################
void runEventAlg::define()
//###################################
{
	//! this method is only executed one at run time
	setDefine(true);

	//! retrieve the algorithms and conditions needed
	m_cReadEvent  = selectionManager::instance()->getCut(m_selRead); 
	m_cWriteEvent = selectionManager::instance()->getCut(m_selWrite);
	

	m_aReadEvent    = processManager::instance()->getAlgorithm(m_comReadEvent);
	m_aSkipEvent    = processManager::instance()->getAlgorithm(m_comSkipEvent);
	m_aWriteEvent   = processManager::instance()->getAlgorithm(m_comWriteEvent);
	m_aProcessEvent = processManager::instance()->getAlgorithm(m_runEvent);

}
//###################################
void runEventAlg::defineOption()
//###################################
{
	//! define the option that the user can replace
	optionVI::defineOption("selRead",&m_selRead);
	optionVI::defineOption("selWrite",&m_selWrite);
	
	optionVI::defineOption("comReadEvent",&m_comReadEvent);
	optionVI::defineOption("comSkipEvent",&m_comSkipEvent);
	optionVI::defineOption("comWriteEvent",&m_comWriteEvent);
	
	optionVI::defineOption("m_runEvent",&m_runEvent);
}
