#include "Event/selectionManager.h"

#include "Event/centellaCuts.h"
#include "Event/userCuts.h"

#include "Event/optionManager.h"
#include "Event/messageManager.h"

selectionManager* selectionManager::m_instance = 0;

//########################################
selectionManager::selectionManager()
//########################################
{
	if (m_instance !=0) return;

	/*! create the server for the cuts and for the cutSelectionTask */
	m_serverCut = new serverVI<cutVI>();
	m_serverSelection = new serverVI<cutSelectionTask>();

	/*! create always two selections. 
	\e selRead to select the events to read.
	\e selWrite to select the events to analyse */
	addCutSelectionTask("selRead", new cutSelectionTask("selRead") );
	addCutSelectionTask("selWrite", new cutSelectionTask("selWrite"));

	m_selName = "";

	m_instance = this;

	/*! get the services */
	setName("selectionManager");
	
	/*! centellaCuts add the system cuts into the server */
	centellaCuts();
	/*! userCuts add the User cuts into ther server */
	userCuts();

}
//########################################
selectionManager::~selectionManager()
//########################################
{
	delete m_serverCut;
	delete m_serverSelection;
}

//########################################
cutVI* selectionManager::getCut(std::string name) const
//########################################
{
	if (m_serverCut->search(name)) return m_serverCut->get(name);
	else  message(" the following cut has not been found in server "+name);
	return 0;
}
//########################################
cutSelectionTask* selectionManager::getCutSelectionTask(std::string name) const
//########################################
{
	if (m_serverSelection->search(name)) return m_serverSelection->get(name);
	message(" the following cutSelectionTask has not been found  in server "+name);
	return 0;
}
//########################################
void selectionManager::addCut(std::string name, cutVI* cut)
//########################################
{
	if (!m_serverCut->search(name)) m_serverCut->add(name, cut);
	else message(" the following cut was already in the server "+name);
}
//########################################
void selectionManager::addCutSelectionTask(std::string name, cutSelectionTask* sel)
//########################################
{
	if (!m_serverCut->search(name)) {
		addCut(name,sel);
		m_serverSelection->add(name, sel);
	} else message(" the following cutSelectionTask was already in the server "+name);
}
//########################################
void selectionManager::setOption(std::string var, std::string content)
//########################################
{
	optionVI::setOption(var,content);
	if (var == "newCutSelectionTask"){
		if (!m_serverSelection->search(content)) {
			addCutSelectionTask(content,new cutSelectionTask(content));
			message(" created cutSelectionTask "+content,"RELEASE");
		} else message(" already existing cutSelectionTask "+content,"RELEASE");
	}
}
//########################################
void selectionManager::defineOption()
//########################################
{
	/*! newCutSelectionTask option will create a cutSelectionTask */
	optionVI::defineOption("newCutSelectionTask",&m_selName);
}
//########################################
void selectionManager::writeOut() const
//########################################
{
	if (!acceptLevel()) return;

	std::ostream& out = messageManager::instance()->out();

	messageVI::writeOut();

	/*! write out the list of cuts */
	if (m_serverCut->size() > 0) {
		out << " -- cuts in cuts server -- " <<"\n";
		out << m_serverCut->nameList();
	}

	/*! write out the list of cutComposites */
	if (m_serverSelection->size() > 0 ) {
		out << " -- cutSelectionTasks in cutSelectionTasks server -- " << "\n";
		out << m_serverSelection->nameList();
	}

	/*! write out information for every cutSelectionTask */
	for (int isel = 0; isel < m_serverSelection->size(); isel++)
		m_serverSelection->get(isel)->writeOut();
}
