#ifndef serviceI_h
#define serviceI_h

#include "Event/optionVI.h"
#include "Event/messageVI.h"

//---------------------------------------------------	
//! Interface to provide the services of the system managers
/*! 
\e serviceI provides services to optionManager.<br>
a \serviceI derived class is a optionVI derived class.<br>
\e serviceI provides services to messageManager.<br>
a \e serviceI derived class is a massageVI derived class.<br>
<ul>
	<li> There is no pure virtual method to be implemented by the derived classes.
	<li> A derived class gets the system services calling the method \c serviceTo().
	Every object that requires a service should have a unique name.
	<li> Via the optionVI services: the derived class should implement the
	\c defineOption() to conver some of its parameters into options 
	(see optionVI for more details). 
	<li> Note that: \c defineOption() is automatically
	called via the \c serviceTo() method. Therefore, the derived class should not
	call \c defineOption().
</ul>
*/
//----------------------------------------------------
// JA Hernando, Santa Cruz, CA, 02/15/00
//----------------------------------------------------

//#########################
class serviceI : public messageVI, public optionVI
//#########################
{
public:

	//! constructor 
	serviceI() {};
	//! default destructor
	~serviceI() {};

	//! call this method to get system services
	void setName(std::string name);

	//! resolve the name ambiguity
	virtual std::string name() const {return messageVI::name();}
};
#endif