
#include "RootTree/convSirClusterList.h"

#include "Event/dataManager.h"
#include "Event/messageManager.h"
#include "RootTree/TkrRecon.h"

//###############################################
void convSirClusterList::define()
//###############################################
{
	if (defined()) return;
	m_sirClusterList = dataManager::instance()->getData("sirClusterList",m_sirClusterList);
	m_Recon          = dataManager::instance()->IO()->treeIn()->core();
	setDefine(true);
}

//###############################################
void convSirClusterList::load() 
//###############################################
{
	if (!defined()) define();

	m_sirClusterList->clear();

    // pointer to the SiCluster TObjArray
   	TObjArray* clusterArr = m_Recon->getTkrRecon()->getSiClusters();
    // loop over all hits 
	for (int ihit = 0; ihit < clusterArr->GetEntries(); ihit ++) {
		TkrSiCluster*    rcl = (TkrSiCluster*) clusterArr->At(ihit);
		sirCluster*       scl = converter(rcl);
		m_sirClusterList->addCluster(scl);
	}

	m_sirClusterList->writeOut();
}
//###############################################
void convSirClusterList::save()
//###############################################
{
	if (!defined()) define();

	int nhits = m_sirClusterList->numClusters();
	unsigned int ihit = 0;
    // pointer to the SiCluster TObjArray
   	TObjArray *clusterArr = m_Recon->getTkrRecon()->getSiClusters();
    // loop over all hits 
	for (ihit = 0; ihit < nhits; ihit ++) {
		sirCluster*    scl = m_sirClusterList->getCluster(ihit);
		TkrSiCluster* rcl = converter(scl);
		clusterArr->Add(rcl);
	}
}
//#######################################
TkrSiCluster* convSirClusterList::converter(sirCluster* scl)
//#######################################
{
	TkrSiCluster* rcl = new TkrSiCluster();
	rcl->setCenterStrip(scl->strip());
	rcl->setId(scl->id());
	rcl->setLayer(scl->plane());
	rcl->setNumStrips(scl->size());
	Point p = scl->position();
	Float_t x = (scl->getView() == sirBase::X? p.x():p.y());
	rcl->setPosition(x);
	TkrSiCluster::TKRAxes axis;
	axis = (scl->getView()== sirBase::X? TkrSiCluster::X : TkrSiCluster::Y);
	rcl->setXY(axis);
	rcl->setZPosition(p.z());
	return rcl;
}
//#######################################
sirCluster* convSirClusterList::converter(TkrSiCluster* rcl)
//#######################################
{

	int id = rcl->getId();
	int iview = rcl->getXY();
	/*! the layers have the different ordering*/
	int ilayer = sirBase::numPlanes()-1 - rcl->getLayer();
	int istrip0 = rcl->getCenterStrip()-0.5*(rcl->getNumStrips()-1);
	int istripf = rcl->getCenterStrip()+0.5*(rcl->getNumStrips()-1);
	int ToT = 0.;
	/*! the sirCluster expect the number of layer*/
	sirCluster* scl = new sirCluster(id,iview,ilayer,istrip0,istripf,ToT);

	double u = rcl->getPosition();
	double z = rcl->getZPosition();
	/*! note that getXY() = 0 for the X view*/
	double x = (!rcl->getXY()?u:0.); 
	double y = (!rcl->getXY()?0:u);
	Point p(x,y,z);
	scl->setPosition(p);
	return scl;
	
}
