#include "RootTree/convSirTrackList.h"
#include "Event/dataManager.h"

//#############################
void convSirTrackList::define()
//#############################
{
	if (defined()) return;
	/*! retrieve the pointers to the data only one at run-time */
	m_tracks = dataManager::instance()->getData("sirTrackList",m_tracks);
	m_clusters = dataManager::instance()->getData("sirClusterList",m_clusters);
	m_recon  = dataManager::instance()->IO()->treeIn()->core();
	setDefine(true);
}
//#############################
void convSirTrackList::load()
//#############################
{
	if (!defined()) define();

	/*! first clear the tracks*/
	m_tracks->clear();

	TkrRecon* Trec = m_recon->getTkrRecon();

	/*! adds the silicon tracks*/
	TObjArray* Ttracks = Trec->getTracks();
	int ntracks = Ttracks->GetEntries();
	for (int itrack = 0 ; itrack < ntracks; itrack++) {
		TkrTrack* Ttrack = (TkrTrack*) Ttracks->At(itrack);
		sirTrack* strack = converter(Ttrack);
		m_tracks->addTrack(strack);
	}

	/*! add the silicon gamma */
	TkrGamma* Tgamma = Trec->getGamma();
	if (Tgamma ==0 ) return;
	sirGamma* sgamma = converter(Tgamma);
	m_tracks->addGamma(sgamma);

}
//################################
sirGamma* convSirTrackList::converter(TkrGamma* Tgamma)
//################################
{
	sirGamma* sgamma = new sirGamma();
	TObjArray* rLocs = Tgamma->getLocator();
	TkrLocator* rloc = (TkrLocator*) rLocs->At(0);
	addLocator(sgamma,rloc);
	TObjArray* Ttracks = Tgamma->getTracks();
	
	int iplane = sirBase::numPlanes();
	double chiSq = 0.;
	double quality = 0.;
	double residual = 0.;
	double eneRec = 0.;
	int ntracks = Ttracks->GetEntries();
	for (int itrack = 0; itrack < ntracks; itrack++) {
		TkrTrack* rtrack = (TkrTrack*) Ttracks->At(itrack);
		int id = rtrack->getId();
		sirTrack* strack = m_tracks->getTrack(id);
		sgamma->addTrack(strack);
		if (iplane > strack->plane()) iplane = strack->plane();
		chiSq += strack->chiSq();
		quality += strack->quality();
		residual += fabs(strack->residual());
		eneRec += strack->energy();
	}
	sgamma->setPlane(iplane);
	sgamma->setEnergy(eneRec/(1.*ntracks));
	sgamma->setChiSq(chiSq/(1.*ntracks));
	sgamma->setQuality(quality/(1.*ntracks));
	sgamma->setResidual(residual/(1.*ntracks));
	return sgamma;
}
//#############################
sirTrack* convSirTrackList::converter(TkrTrack* rtrack)
//#############################
{
	sirTrack* strack = new sirTrack();

	double chi2   = rtrack->getChi2();
	double quality= rtrack->getQuality();
	double res    = rtrack->getResidual();

	/*! set the sirQuality of the track*/
	strack->setChiSq(chi2);
	strack->setQuality(quality);
	strack->setResidual(res);

	/*! add the locator of the track */
	TObjArray* rLocs = rtrack->getLocator();
	TkrLocator* rloc = (TkrLocator*) rLocs->At(0);
	addLocator(strack,rloc);
	double eneRec = rtrack->getEnergyDet();
	strack->setEnergy(eneRec);

	/*! set the sirID of the locator */
	int fstLayer = rtrack->getFirstLayer();
	strack->setPlane(fstLayer);
	int id = rtrack->getId();
	strack->setId(id);
	double eneIn  = rtrack->getEnergyInput();
	strack->setInputEnergy(eneIn);
	if (rloc->getSlopeX()== 0.) strack->setView(sirBase::Y);
	else if (rloc->getSlopeY() == 0.) strack->setView(sirBase::X);
	else strack->setView(sirBase::XY);

	/*! add the hits to the track*/
	TObjArray* rHits = rtrack->getHits();
	int nHits = rHits->GetEntries();
	for (int ihit = 0; ihit < nHits ; ihit++) {
		TkrHit* rhit = (TkrHit*) rHits->At(ihit);
		sirHit* shit = converter(rhit);
		shit->setId(ihit);
		strack->addHit(shit);
	}

	return strack;
}
//#############################
sirHit* convSirTrackList::converter(TkrHit* rhit)
//#############################
{
	sirHit* shit = new sirHit();

	/*! set the quality of the sirHit */
	double chi2 = rhit->getChi2();
	double res = rhit->getResidual();
	shit->setChiSq(chi2);
	shit->setResidual(res);

	TkrLocator* rLoc = rhit->getLocator();
	addLocator(shit,rLoc);
	/*! find the cluster by the id*/
	TkrSiCluster* rcluster = rhit->getSiCluster();
	int id = rcluster->getId();
	shit->setCluster(m_clusters->getCluster(id));

	/*! set the Id of the sirHit from the cluster */
	shit->setView(shit->getCluster()->getView());
	shit->setPlane(shit->getCluster()->plane());

	return shit;
}
//#############################
void convSirTrackList::addLocator(sirLocator* loc, TkrLocator* rloc)
//#############################
{
	double slopex = rloc->getSlopeX();
	double slopey = rloc->getSlopeY();
	double modulo = sqrt(slopex*slopex+slopey*slopey+1.); 
	loc->setDirection(-1.*slopex/modulo,-1.*slopey/modulo,-1./modulo);
	double xpos = rloc->getPosition().X();
	double ypos = rloc->getPosition().Y();
	double zpos = rloc->getPosition().Z();
	loc->setPosition(xpos,ypos,zpos);
	double errx      = rloc->getSigma_X();
	double errxslope = rloc->getSigma_Xslope();
	double errxcov   = rloc->getCov_X();
	loc->setXcov(errx,errxslope,errxcov);
	double erry      = rloc->getSigma_Y();
	double erryslope = rloc->getSigma_Yslope();
	double errycov   = rloc->getCov_Y();
	loc->setYcov(erry,erryslope,errycov);
}