#ifndef converterImport_h
#define convertetImport_h

#include "Event/converterVI.h"
#include "Event/defineVI.h"
#include "Event/dataManager.h"

//--------------------------------
//! Import to converterVI a host class that operates the conversion over some visiting classes
//--------------------------------
// JA Hernando, Santa Cruz, CA, 06/16/00
//--------------------------------
//################################
template<class Thost, class Tvisitor> class converterImport : public converterVI, public defineVI
//################################
{
public:

	//! constructor - set the name of the visiting class and the visitor template class
	converterImport(Thost* th, std::string name):m_Thost(th),m_name(name) {m_Tvisitor = 0;}
	//! default destructor
	~converterImport() {}

	//! define - get the pointer to template visitor class from the dataManager
	void define() {
		m_Tvisitor = dataManager::instance()->getData(m_name,m_Tvisitor); 
		setDefine(true);
	}
	//! load the visitor template class using the host template class method \c load()
	virtual void load() {
		if (!defined()) define(); 
		m_Tvisitor->clear(); 
		m_Thost->load(m_Tvisitor);
		m_Tvisitor->writeOut();
	}
	//! save the visitor template class using the save template class method \c save()
	virtual void save() {
		if (!defined()) define(); 
		m_Thost->save(m_Tvisitor);
	}

private:

	//! name of the visiting template class
	std::string m_name;
	//! pointer to the host template class
	Thost* m_Thost;
	//! pointer to the visitor template class
	Tvisitor* m_Tvisitor;
};
#endif