#include "RootTree/hFolderSiToT.h"

#include "Event/dataManager.h"

#include "TROOT.h"
#include "TFile.h"
#include "TObject.h"
#include "TObjArray.h"
#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"

//####################################
void hFolderSiToT::ini()
//####################################
{
	m_siLayers = 0;
	m_tracks   = 0;
	m_clusters = 0;

	m_tagger     = 0;
	m_calCluster = 0;

}
//####################################
void hFolderSiToT::define()
//####################################
{
	if (defined()) return;

	m_siLayers = dataManager::instance()->getData("SiLayers",m_siLayers);
	m_tracks   = dataManager::instance()->getData("sirTrackList",m_tracks);
	m_clusters = dataManager::instance()->getData("csiCluster",m_clusters);

	m_tagger     = dataManager::instance()->getData("BeamTagger",m_tagger);
	m_calCluster = dataManager::instance()->getData("csiCluster",m_calCluster);


	getFile()->cd();

	newHisto("H1","htXToTGamma"," ToT Gamma x view",50,0,100);
	newHisto("H1","htYToTGamma"," ToT Gamma y view",50,0,100);
	book();

	char* varname = "idevt:eneFit:eneTag:xToT:yToT:xplane:yplane:xpos:ypos:zpos:xcos:ycos:zcos";
	TNtuple* nt10 = new TNtuple("ntToTGamma"," ToT for the gamma ",varname);

	char* varname2 = "idevt:plane:view:ToT";
	TNtuple* nt20 = new TNtuple("ntToTPlanes"," ToT for the planes",varname2);

	setDefine(true);
}
//####################################
void hFolderSiToT::fill()
//####################################
{
	if (!defined()) define();

	fillToTGamma();
	fillToTPlanes();
}
//####################################
void hFolderSiToT::fillToTGamma()
//####################################
{
	getFile()->cd();

	Float_t idevt = dataManager::instance()->IO()->eventNumber();
	Float_t eneFit = m_calCluster->eneFit();
	Float_t eneTag = -1.;
	if (m_tagger->nParticle() >0) eneTag = 1000.*m_tagger->momentum(0);
	sirGamma* gamma = m_tracks->gamma();
	if (!gamma) return;
	
	Float_t xplane = -1.;
	Float_t xToT = -1.;
	if (gamma->numXTracks() >0) {
		sirTrack* xtrack = gamma->getTrack(sirBase::X, 0);
		xplane = xtrack->plane();
		int ilayer = 15-xplane;
		SiLayer* xlayer = m_siLayers->getSiLayer(ilayer,sirBase::viewToInt(sirBase::X));
		xToT = xlayer->ToT();
	}
	Float_t yplane = -1.;
	Float_t yToT = -1.;
	if (gamma->numYTracks() >0) {
		sirTrack* ytrack = gamma->getTrack(sirBase::Y, 0);
		yplane = ytrack->plane();
		int ilayer = 15-yplane;
		SiLayer* ylayer = m_siLayers->getSiLayer(ilayer,sirBase::viewToInt(sirBase::Y));
		yToT = ylayer->ToT();
	}
	
	Float_t xpos = gamma->position().x();
	Float_t ypos = gamma->position().y();
	Float_t zpos = gamma->position().z();
	Float_t xcos = gamma->direction().x();
	Float_t ycos = gamma->direction().y();
	Float_t zcos = gamma->direction().z();
		
	((TH1F*) getFile()->Get("htXToTGamma"))->Fill(xToT);
	((TH1F*) getFile()->Get("htYToTGamma"))->Fill(yToT);
	((TNtuple*) getFile()->Get("ntToTGamma"))->
		Fill(idevt,eneFit,eneTag,xToT,yToT,xplane,yplane,xpos,ypos,zpos,xcos,ycos,zcos);
}
//####################################
void hFolderSiToT::fillToTPlanes()
//####################################
{
	getFile()->cd();

	Float_t idevt = dataManager::instance()->IO()->eventNumber();

	int nLayers = m_siLayers->num();
	for (int ilayer = 0; ilayer < nLayers; ilayer++ ) {
		SiLayer* silayer = m_siLayers->Layer(ilayer);
		// layers have hardware ID (0=bottom) planes have rec ID (0=top) 
		Float_t rplane = 15 - silayer->layer();
		Float_t rview = (float) silayer->view();
		Float_t rToT = silayer->ToT();
		((TNtuple*) getFile()->Get("ntToTPlanes"))->
		Fill(idevt,rplane,rview,rToT);
	}
}
