#include "RootTree/hFolderSirClusterList.h"
//###################################
#include "RootTree/hFolderSirTrackList.h" //edited for cluster vs tracks
//######################################
#include "Event/dataManager.h"

#include "TROOT.h"
#include "TFile.h"
#include "TObject.h"
#include "TObjArray.h"
#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"
#include "TProfile.h"

//###########################
void hFolderSirClusterList::define()
//###########################
{
	/*! get the pointer to the data*/
	m_clusters = dataManager::instance()->getData("sirClusterList",m_clusters);

//########################
//edited for cluster vs tracks and strip histograms
//########################
	m_ctracks = dataManager::instance()->getData("sirTrackList",m_ctracks); //edited for cluster vs tracks
//#############################

	/*! go to this ROOT file */
	getFile()->cd();

	newHisto("H1","htNumClusters"," number of clusters ",200,0,200);
	newHisto("H2","stXNumClusters"," X Plane vs. #clusters ",16,0,16,50,0,50);
	newHisto("H2","stYNumClusters"," Y plane vs. #clusters ",16,0,16,50,0,50);
	newHisto("HP","htXSizeClustersPlane","Plane vs. avg size of the X clusters",20,0,20);
	newHisto("HP","htYSizeClustersPlane","Plane vs. avg size of the Y clusters",20,0,20);
	newHisto("H2","stXSizeClustersPlane"," plane vs size of the X clusters ",16,0,16,30,0,30);
	newHisto("H2","stYSizeClustersPlane"," plane vs size of the Y clusters ",16,0,16,30,0,30);
	newHisto("H2","stXZClusters"," XZ of the clusters",50,-25,25,25,0,50);
	newHisto("H2","stYZClusters"," YZ of the clusters",50,-25,25,25,0,50); 
	newHisto("HP","htXNumClusteravg"," # Clusters vs. X plane average ",20,0,20);
	newHisto("HP","htYNumClusteravg"," # Clusters vs. Y plane average ",20,0,20);
	newHisto("HP","htXNumStripavg"," # Strips vs. X plane average ",20,0,20);
	newHisto("HP","htYNumStripavg"," # Strips vs. Y plane average ",20,0,20);

//########################
//edited for cluster vs tracks and strip histograms
//########################
	newHisto("H2","stNumClusterNumTrack","#Clusters vs. #Tracks",197,3,200,14,1,15);
	newHisto("HP","htClusterTrack"," # Clusters vs. #Tracks average ",190,10,200);
	newHisto("H2","stXStripPlane"," Plane vs. Strip id X  ",1600,0,1600,16,0,16);
	newHisto("H2","stYStripPlane"," Plane vs. Strip id Y  ",1600,0,1600,16,0,16);

	/*! studying number of strips per plane and size cluster per plane*/
	newHisto("H2","stXNumStripsPlane"," Plane vs. num strips X ",16,0,16,30,0,30);
	newHisto("H2","stYNumStripsPlane"," Plane vs. num strips Y ",16,0,16,30,0,30);
	newHisto("H2","stXNumStripsNumClusters"," Num clusters vs. num strips X ",30,0,30,30,0,30);
	newHisto("H2","stYNumStripsNumClusters"," Num clusters vs. num strips Y ",30,0,30,30,0,30);

//define ntuple for cluster AND track info
	char* var1 = "idevt:nxclusters:nyclusters:nxtracks:nytracks:nxstrips:nystrips:plane";
	TNtuple* nt10 = new TNtuple("ntcluster"," ntuple with cluster info ",var1);
//#####################

	/*! the user can redefine the histos by its name in myHistos.in file */
//	setBookOptionFile("../myHistos.in");
//	redefine();
	/*! now book the rHistos */
	book();

	/* histogram def */
	setDefine(true);
}
//###########################
void hFolderSirClusterList::fill()
//###########################
{

	/*! if the histograms are not defined, define them */
	if (!defined()) define();

	Int_t nclusters = m_clusters->numClusters();
	if (nclusters == 0) return;
//########################
//edited for cluster vs tracks and strip histograms
//########################
	int idevt = dataManager::instance()->IO()->eventNumber();
	Float_t nXtracks = m_ctracks->numXTracks();
	Float_t nYtracks = m_ctracks->numYTracks();
	Float_t nctracks = m_ctracks->numTracks(); //edited for cluster vs tracks
	((TH2F*) getFile()->Get("stNumClusterNumTrack"))->Fill(nclusters,nctracks);
	((TProfile*) getFile()->Get("htClusterTrack"))->Fill(nclusters,nctracks);
//#########################

	((TH1F*) getFile()->Get("htNumClusters"))->Fill(nclusters);
	
	for (int icl = 0; icl < nclusters; icl++) {
		sirCluster* cl = m_clusters->getCluster(icl); 
		Float_t size = cl->size();
		Float_t x = cl->position().x();
		Float_t y = cl->position().y();
		Float_t z = cl->position().z();
		Float_t plane = cl->plane();
		Float_t strip = cl->strip(); 
		sirBase::VIEW v = cl->getView();
		if (v == sirBase::X) {
			((TH2F*) getFile()->Get("stXStripPlane"))->Fill(strip,plane);
			((TProfile*) getFile()->Get("htXSizeClustersPlane"))->Fill(plane,size);
			((TH2F*) getFile()->Get("stXZClusters"))->Fill(x,z);

			((TH2F*) getFile()->Get("stXSizeClustersPlane"))->Fill(plane,size);

		} else if (v == sirBase::Y) {
			((TH2F*) getFile()->Get("stYStripPlane"))->Fill(strip,plane);
			((TProfile*) getFile()->Get("htYSizeClustersPlane"))->Fill(plane,size);
			((TH2F*) getFile()->Get("stYZClusters"))->Fill(y,z);
			
			((TH2F*) getFile()->Get("stYSizeClustersPlane"))->Fill(plane,size);

		}
	
	}

	for (int iplane = 0; iplane < sirBase::numPlanes(); iplane++) {
		Float_t nx = m_clusters->numClusters(sirBase::X, iplane);
		Float_t ny = m_clusters->numClusters(sirBase::Y, iplane);
		Float_t nxstrips = 0;
		Float_t nystrips = 0;
		const std::vector<sirCluster*>& xlist = m_clusters->getClusters(sirBase::X,iplane);
		const std::vector<sirCluster*>& ylist = m_clusters->getClusters(sirBase::Y,iplane);
		for (int icl = 0; icl < nx; icl++) { nxstrips += xlist[icl]->size();}
		for (icl = 0; icl < ny; icl++) { nystrips += ylist[icl]->size();}

		Float_t jplane = 1.* iplane;
		
		if (nxstrips > 0) {
			((TH2F*) getFile()->Get("stXNumStripsPlane"))->Fill(jplane,nxstrips);
			((TH2F*) getFile()->Get("stXNumStripsNumClusters"))->Fill(nx,nxstrips);	
			((TH2F*) getFile()->Get("stXNumClusters"))->Fill(jplane,nx);
			((TProfile*) getFile()->Get("htXNumClusteravg"))->Fill(jplane,nx);	
			((TProfile*) getFile()->Get("htXNumStripavg"))->Fill(jplane,nxstrips);
		}
		
		if (nystrips > 0) {
			((TH2F*) getFile()->Get("stYNumStripsPlane"))->Fill(jplane,nystrips);
			((TH2F*) getFile()->Get("stYNumStripsNumClusters"))->Fill(ny,nystrips);
			((TH2F*) getFile()->Get("stYNumClusters"))->Fill(jplane,ny);
			((TProfile*) getFile()->Get("htYNumClusteravg"))->Fill(jplane,ny);
			((TProfile*) getFile()->Get("htYNumStripavg"))->Fill(jplane,nystrips);
		}
	
		//########################
		//edited for cluster vs tracks and strip histograms
		//########################
		((TNtuple*) getFile()->Get("ntcluster"))->Fill(idevt,
				nx,ny,nXtracks,nYtracks,nxstrips,nystrips,jplane); //edited for cluster vs tracks
		//#########################

	}
}