#include "RootTree/hFolderSirTrackList.h"

#include "Event/dataManager.h"
#include "Event/selectionManager.h"

#include "TROOT.h"
#include "TFile.h"
#include "TObject.h"
#include "TObjArray.h"
#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"
#include <math.h>


//###########################
void hFolderSirTrackList::define()
//###########################
{
	/*! get the pointers to the data*/
	m_tracks = dataManager::instance()->getData("sirTrackList",m_tracks);
	m_tagger = dataManager::instance()->getData("BeamTagger",m_tagger);
	m_ievt = -1;
	m_itrack = 0;

	getFile()->cd();

	defineGeneral();
	defineGamma();
	defineTracks();
	
	/*! the user can redefine the histos by its name in myHistos.in file */
//	setBookOptionFile("../myHistos.in");
//	redefine();
	book();

	setDefine(true);
}

//###########################
void hFolderSirTrackList::fill()
//###########################
{

	/*! if the histograms are not defined, define them */
	if (!defined()) define();

	m_ievt = dataManager::instance()->IO()->eventNumber();

	fillGeneral();
	fillGamma();
	fillTracks();
}

//###########################
void hFolderSirTrackList::defineGeneral()
//###########################
{
	/*! book general track histograms */
	newHisto("H1","htNumTracks"," number of Tracks ",15,1,16);
	newHisto("H1","htXNumTracks"," X Tracks",9,0,9);
	newHisto("H1","htYNumTracks"," Y Tracks",9,0,9);
	newHisto("H2","stXYNumTracks","X vs Y tracks",9,0,9,9,0,9);
}
//###########################
void hFolderSirTrackList::defineGamma()
//###########################
{
	/*! book histograms for gamma */
	newHisto("H1","htgamNumTracks"," tracks of gamma",9,0,9);
	newHisto("H1","htgamXNumTracks"," X tracks of gamma",9,0,9);
	newHisto("H1","htgamYNumTracks"," Y tracks of gamma",9,0,9);
	newHisto("H1","htgamPlane"," 1st plane of gamma",16,0,16);
	newHisto("H1","htgamChiSq"," chiSq of gamma",80,0,40);
	newHisto("H1","htgamQuality"," quality of gamma",80,0,40);
	newHisto("H1","htgamResidual"," residual plane of gamma",40,-1,1);
	newHisto("H1","htgamEneIn"," input ene of gamma",60,0,15);
	newHisto("H1","htgamEneRec"," rec ene of gamma",60,0,15);
	newHisto("H1","htgamXdir"," x direction of gamma",800,-1.,1.);
	newHisto("H1","htgamYdir"," y direction of gamma",800,-1.,1.);
	newHisto("H1","htgamXpos"," x position of gamma",1617,-16,17);
	newHisto("H1","htgamYpos"," y position of gamma",1617,-16,17);
	newHisto("H1","htgamZpos"," z position of gamma",60,0,60);
	
	char* var1 = "idevt:nxtracks:nytracks:plane:chiSq:enetag:enein:enerec:xtan:ytan:zcos:theta:xpos:ypos:zpos";
	TNtuple* nt10 = new TNtuple("ntgamma"," ntuple with gamma info ",var1);
}
//###########################
void hFolderSirTrackList::defineTracks()
//###########################
{
	newHisto("H1","httkrView"," view ",3,0,3);
	newHisto("H1","httkrPlane"," 1st plane of track",16,0,16);
	newHisto("H1","httkrNumHits"," number of hits of track",18,0,18);
	newHisto("H1","htXtkrNumHits"," number of X hits of track",18,0,18);
	newHisto("H1","htYtkrNumHits"," number of Y hits of track",18,0,18);
	newHisto("H1","httkrEneIn"," input ene of track",60,0,15);
	newHisto("H1","httkrEneRec"," rec ene of track",60,0,15);
	newHisto("H1","httkrChiSq"," chiSq of track",80,0,40);
	newHisto("H1","httkrQuality"," quality of track",80,0,40);
	newHisto("H1","httkrResidual"," residual of track",40,0,2);
	newHisto("H1","httkrXdir"," x direction of track",800,-1.,1.);
	newHisto("H1","httkrYdir"," y direction of track",800,-1.,1.);
	newHisto("H1","httkrXpos"," x position of track",1617,-16,17);
	newHisto("H1","httkrYpos"," y position of track",1617,-16,17);
	newHisto("H1","httkrZpos"," z position of track",60,0,60);
	
	char* var2 = "idevt:idtrack:view:plane:nhits:chisq:quality:enein:enerec:xtan:ytan:zcos:xpos:ypos:zpos";
	TNtuple* nt20 = new TNtuple("nttracks"," ntuple with tracks info ",var2);
}
//###########################
void hFolderSirTrackList::fillGeneral()
//###########################
{
	Float_t ntracks  = m_tracks->numTracks();
	Float_t nXtracks = m_tracks->numXTracks();
	Float_t nYtracks = m_tracks->numYTracks();

	((TH1F*) getFile()->Get("htNumTracks"))->Fill(ntracks);
	((TH1F*) getFile()->Get("htXNumTracks"))->Fill(nXtracks);
	((TH1F*) getFile()->Get("htYNumTracks"))->Fill(nYtracks);
	((TH2F*) getFile()->Get("stXYNumTracks"))->Fill(nXtracks,nYtracks);
}

//###########################
void hFolderSirTrackList::fillGamma()
//###########################
{
	sirGamma* gamma = m_tracks->gamma();
	if (gamma ==0) return;

	Float_t ntracks = gamma->numTracks();
	Float_t nXtracks= gamma->numXTracks();
	Float_t nYtracks= gamma->numYTracks();
	Float_t stPlane = gamma->plane();
	Float_t chiSq   = gamma->chiSq();
	Float_t quality = gamma->quality();
	Float_t res     = gamma->residual();
	Float_t eneIn   = gamma->inputEnergy();
	Float_t eneRec  = gamma->energy();
	Float_t xtan    = gamma->slopeX();
	Float_t ytan    = gamma->slopeY();
	Float_t zcos	= gamma->direction().z();
	Float_t theta	= acos(-1.*zcos)*(180./3.1415);
	Float_t xpos    = gamma->X();
	Float_t ypos    = gamma->Y();
	Float_t zpos    = gamma->Z();

	((TH1F*) getFile()->Get("htgamNumTracks"))->Fill(ntracks);
	((TH1F*) getFile()->Get("htgamXNumTracks"))->Fill(nXtracks);
	((TH1F*) getFile()->Get("htgamYNumTracks"))->Fill(nYtracks);
	((TH1F*) getFile()->Get("htgamPlane"))->Fill(stPlane);
	
	((TH1F*) getFile()->Get("htgamChiSq"))->Fill(chiSq);
	((TH1F*) getFile()->Get("htgamQuality"))->Fill(quality);
	((TH1F*) getFile()->Get("htgamResidual"))->Fill(res);

	((TH1F*) getFile()->Get("htgamEneIn"))->Fill(eneIn);
	((TH1F*) getFile()->Get("htgamEneRec"))->Fill(eneRec);

	((TH1F*) getFile()->Get("htgamXdir"))->Fill(xtan);
	((TH1F*) getFile()->Get("htgamYdir"))->Fill(ytan);
	((TH1F*) getFile()->Get("htgamXpos"))->Fill(xpos);
	((TH1F*) getFile()->Get("htgamYpos"))->Fill(ypos);
	((TH1F*) getFile()->Get("htgamZpos"))->Fill(zpos);
	
	Float_t eneTag = -1.;
	if (m_tagger->nParticle() ==1) eneTag = 1000.*m_tagger->momentum(0);

	((TNtuple*) getFile()->Get("ntgamma"))->Fill(m_ievt,
			nXtracks, nYtracks, stPlane,
			chiSq, eneTag,
			eneIn, eneRec,
			xtan, ytan, zcos, theta, xpos, ypos, zpos);
}
//###########################
void hFolderSirTrackList::fillTracks()
//###########################
{
	m_itrack= 0;
	int ntracks = m_tracks->numTracks();
	for (int itrack = 0; itrack < ntracks; itrack++) {
		sirTrack* track = m_tracks->getTrack(itrack);
		fillTrack(track);
	}
}
//###########################
void hFolderSirTrackList::fillTrack(sirTrack* track)
//###########################
{
	m_itrack++;
	Float_t stPlane = track->plane();
	Float_t tview   = track->getView();
	Float_t nHits   = track->numHits();
	Float_t chiSq   = track->chiSq();
	Float_t quality = track->quality();
	// Float_t res     = track->residual();
	Float_t eneIn   = track->inputEnergy();
	Float_t eneRec  = track->energy();
	Float_t xtan    = track->slopeX();
	Float_t ytan    = track->slopeY();
	Float_t zcos	= track->direction().z();
	Float_t xpos    = track->X();
	Float_t ypos    = track->Y();
	Float_t zpos    = track->Z();

	((TH1F*) getFile()->Get("httkrPlane"))->Fill(stPlane);
	((TH1F*) getFile()->Get("httkrView"))->Fill(tview);
	((TH1F*) getFile()->Get("httkrNumHits"))->Fill(nHits);

	((TH1F*) getFile()->Get("httkrChiSq"))->Fill(chiSq);
	((TH1F*) getFile()->Get("httkrQuality"))->Fill(quality);
//	((TH1F*) getFile()->Get("httkrResidual"))->Fill(res);
	
	((TH1F*) getFile()->Get("httkrEneIn"))->Fill(eneIn);
	((TH1F*) getFile()->Get("httkrEneRec"))->Fill(eneRec);

	if (track->getView() != sirBase::Y) {
		((TH1F*) getFile()->Get("httkrXdir"))->Fill(xtan);
		((TH1F*) getFile()->Get("httkrXpos"))->Fill(xpos);
		((TH1F*) getFile()->Get("htYtkrNumHits"))->Fill(nHits);
	} else if (track->getView() != sirBase::X) {
		((TH1F*) getFile()->Get("httkrYdir"))->Fill(ytan);
		((TH1F*) getFile()->Get("httkrYpos"))->Fill(ypos);
		((TH1F*) getFile()->Get("htXtkrNumHits"))->Fill(nHits);
	}
	((TH1F*) getFile()->Get("httkrZpos"))->Fill(zpos);
	
	((TNtuple*) getFile()->Get("nttracks"))->Fill(m_ievt,m_itrack,
		tview,stPlane,nHits,
		chiSq,quality,
		eneIn,eneRec,
		xtan,ytan,zcos,xpos,ypos,zpos);
}