#include "rTreeRawServer.h"
#include "ACDTile.h"
#include "CalHit.h"
#include "StripID.h"
#include "TkrLayer.h"
#include "Tagger.h"

#include "TTree.h"

#include "RootTree/converterImport.h"

//#######################################
void rTreeRawServer::defineConverters()
//#######################################
{
	/*! add the converter from rawEventID */
	addConverter("rawEventID",	 
		new converterImport<rTreeRawServer,eventID>(this,"rawEventID") );

	/*! add the converter for the BeamTagger */
	addConverter("BeamTagger",	 
		new converterImport<rTreeRawServer,BeamTagger>(this,"BeamTagger") );
	
	/*! add the converter to SiLayers */
	addConverter("SiLayers",  
		new converterImport<rTreeRawServer,SiLayers>(this,"SiLayers") );
	
	/*! create the load and fill algorithms with the converters */
	makeAlgorithms();
}
//--------------------------------------------------------------

//###############################################
void rTreeRawServer::load(eventID* rawEvt)
//###############################################
{
	int evt = core()->getEvent();
	rawEvt->setEventID(evt);
	int run = core()->getRun();
	rawEvt->setRunID(run);
}
//###############################################
void rTreeRawServer::load(BeamTagger* tagger)
//###############################################
{	
	ESAPID* pid = core()->getTagger()->getPID();

	// beam info
	tagger->setClean(pid->getClean());
	tagger->setXBeam(pid->getXbeam());
	tagger->setYBeam(pid->getYbeam());
	//Quality info
	tagger->setGoodG(pid->getOneGoodG());
	tagger->setGoodE(pid->getOneGoodE());	
	tagger->setGoodH(pid->getOneGoodH());	
	tagger->setGoodP(pid->getOneGoodP());
	tagger->setGoodPi(pid->getOneGoodPi());	
	tagger->setGoodK(pid->getOneGoodK());	

	// particle info
	int nPart = pid->getNpart();
	for (int i = 0; i < nPart; i++) {
		tagger->addParticle((int)pid->getIDN(i), (double)pid->getMomN(i));
	}
}

//###############################################
void rTreeRawServer::load(SiLayers* siLayers)
//###############################################
{
	int OFFSET = 64;   // threre is an offser in the idaddress
	int nTKR = core()->getTKR()->GetEntries();
	for (int i = 0; i < nTKR; i++) {
		
		TkrLayer* rLayer = (TkrLayer*) core()->getTKR()->At(i);
		int XY = (rLayer->getXY() == TkrLayer::X ? 1 : 0);
		double tot = rLayer->getTOT(XY);
		//double tot = 0;
		
		int nStrips = rLayer->getStrips()->GetEntries();
		if (nStrips >0 ) {
			SiLayer* siLayer = new SiLayer(rLayer->getLayer(), XY, tot);
			StripID* rStrip;
			for (int iStrip=0; iStrip < nStrips; iStrip++) {
				rStrip = (StripID*) rLayer->getStrips()->At(iStrip);
				int iaddress = rStrip->getID() - OFFSET;
				siLayer->addStrip((int) iaddress);
			}
			siLayers->add(siLayer);
		}
	}
}
