#ifndef rTreeRawServer_h
#define rTreeRawServer_h

#include "Event.h"

#include "Event/rTreeBase.h"
#include "Event/converterServer.h"

#include "reconstruction/BeamTagger.h"
#include "reconstruction/eventID.h"
#include "reconstruction/SiLayers.h"

//############################################
class rTreeRawServer : public rTreeBase<Event>, public converterServer
//#############################################
{	
public:

	//! constructor: create the converters
	/*! the constructor of converterServer will call defineConverters*/
	rTreeRawServer():rTreeBase<Event>(new Event(),"Event") {defineConverters();}
	//! default destructor
	~rTreeRawServer(){};

	//! load Raw data into rawEventID
	/*! the run and event ID */
	void load(eventID* rawEvt); 
	//! export Raw data into "BeamTagger" transient data class
	/*! data with the beam information provided by the tagger */
	void load(BeamTagger* tagger); 
	//! export SiLayers information into transiet data */
	void load(SiLayers* silayers);

	//! null save rawEventID
	void save(eventID* rawEvt) {}; 
	//! export Raw data into "BeamTagger" transient data class
	void save(BeamTagger* tagger) {}; 
	//! null save SiLayers
	void save(SiLayers* silayers) {}

protected:

	//! The User should define in this methods the converters from Recon->Transient
	/*! The system will not know the persistent data and will interact only
	with the algorithms created for the converters that the User declare here */
	virtual void defineConverters();
};
#endif

