
#ifndef rTreeReconServer_h
#define rTreeReconServer_h

#include "Event/rTreeBase.h"
#include "Event/converterServer.h"

#include "Recon.h"
//--------------------------------------------------------------------------
//! Server for the Recon Tree
/*!
<ul> 
	<li> This server handles the persistent data stored in a ROOT tree. This
	server should handle: 
	<ul>
		<li> A IOfileVI implemetation (i.e. \c readEvent() \c writeEvent() etc)
		<li> A converterServer that will implement the converstion into loadAlg and
		saveAlg algorithms available.
	</ul>
	<li> All the basic file (IOfileVI) operations are handle by rTreeBase.
The rTreeBase three should be implemented given the core (via template) or main branch
of the ROOT tree. Also the name of the class (m_class) should be indicate in
the constructor.
	<li> The User should make the converters between the rootTree and
the transient classes, and added in the method \c defineConverters(). The
converterServer class will automatically create the load and save algorithms
	<li> The name of the converters should be the same of the transient class
to which it converts the persistency data
</ul>
*/
//---------------------------------------------------------------------------
//############################################
class rTreeReconServer : public rTreeBase<Recon>, public converterServer
//#############################################
{	
public:

	//! constructor: create the converters
	/*! the constructor of converterServer will call defineConverters*/
	rTreeReconServer():rTreeBase<Recon>(new Recon(),"Recon") {defineConverters();}

protected:

	//! The User should define in this methods the converters from Recon->Transient
	/*! The system will not know the persistent data and will interact only
	with the algorithms created for the converters that the User declare here */
	virtual void defineConverters();
};
#endif

