//     $Id: EventLoop.cxx,v 1.1.1.1 2000/04/03 21:03:48 wilko Exp $
// Project: Atlas top-level prototype
//  Author: Toby Burnett
//
//
#include "EventLoop.h"
#include <string>
#include "control/CompoundCommand.h"
#include "control/SimpleCommand.h"
#include "control/Menu.h"


EventLoop* EventLoop::s_running = 0;

//////////////////////////////////////////////////////////////////////////
//                set loop control command

void EventLoop::set_loop_count()
{
    int count = getCount();
    m_menu.query("Number of Events?", & count);
    setCount(count);
}

void EventLoop::single_event()
{
    if( status()==RUNNING ) pause();
    else {
        int old_count = getCount();
        Status old_status = status();
        setCount(getCurrent()+1);
        pause();
        resume();
        setCount(old_count);
        if( old_status==LoopCommand::PAUSED)pause();
    }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
EventLoop::EventLoop( Menu& menu, Command* trig)
    :LoopCommand(trig, 100000),
     m_menu(menu)
{
    if(trig) setup(trig);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
EventLoop::EventLoop( Command* trig)
// deprectated constructor
    : LoopCommand(trig, 100000),
      m_menu(*Menu::instance())
{
    if(trig) setup(trig);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void
EventLoop::setup(Command* trig)
// private setup stuff
{
    LoopCommand::setCommand(trig);

    // now make a command that also checks messages after an event
    CompoundCommand* trig2 = new CompoundCommand(trig);
    trig2->append(new SimpleCommand<Menu>(&m_menu, &Menu::check_messages)); 
    setCommand(trig2);


    // start a special pull-down menu
    SubMenu& sub = m_menu.sub_menu("Event Loop");
    m_sub_menu = &sub; // for external access
    
    // create special button that generates a single event, but which will increment
    // the event count if the status is paused
    Command* one_event = new SimpleCommand<EventLoop>(this,&EventLoop::single_event);
    sub.add_button("One event", one_event);

    sub.add_button("Start", this );

    sub.add_button("Stop",  abortCommand());
    sub.add_button("Pause", pauseCommand());

    Command* resume = new SimpleCommand<EventLoop>(this,&EventLoop::execute);
    sub.add_button("Resume",resume);
    sub.add_separator();
    sub.add_button("Set Number of Events", new SimpleCommand<EventLoop>(this,&EventLoop::set_loop_count) );

    // now set up some useful key-stroke shortcurs
    m_menu.register_key(' ', one_event);// space bar -- single event (or pause run)
    m_menu.register_key('\r',resume);	// cr -- start or resume

}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void EventLoop::execute()
{
    if( status()==RUNNING ) return;

    s_running = this;
    if( status()!=PAUSED) LoopCommand::execute();
    else LoopCommand::resume();
    s_running = 0;
}
SubMenu& EventLoop::menu(){return *m_sub_menu;}

