//     $Id: PrintControl.cxx,v 1.2 2000/05/11 16:54:04 terrier Exp $
// Project: Atlas top-level prototype
//  Author: Toby Burnett
//
//
// Implement the PrintControl class 
//
// When this class has been set up, new print commands can be added to the menu
// by:      
//      PrintControl::instance()->addItem("my printout", myCommand);
//
// where
//    myCommand is an instance of a subclass of Command that will output to
// the std::ostream* PrintControl::instance()->out(); 

#include "control/PrintControl.h"
#include "control/Messenger.h"
#include "control/SimpleCommand.h"
#include <assert.h>


//////////////////////////////////////////////////////////////////////////////
//   Constructor: set up menu, get the GUI's text window to print to
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PrintControl::PrintControl(std::ostream* out)
// deprecated constructor
    : m_print(out),
      m_menu(*Menu::instance())
{
    setup();
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PrintControl::PrintControl(Menu& menu, std::ostream*out )
    : m_print(out), 
      m_menu(menu)
{
    setup();
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PrintControl::~PrintControl()
{
    PrintList::iterator it = m_print_commands.begin(); 
    while (it !=  m_print_commands.end()) 
	delete (*it++);

}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PrintCommand::~PrintCommand()
{ 
    delete m_cmd; 
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void PrintControl::setup()
{
    s_instance = this;
    disable();
    // create Print pull-down menu
    m_sub_menu = &m_menu.sub_menu("Print");
    // schedule call back to finishSetup method
    m_menu.add(new MenuClient<PrintControl>(this));

    messenger.stream(*m_print);
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// file scope class that implements the command, followed by a flush

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void PrintControl::add(Command* cmd, const std::string& title,  bool enabled)
{

    PrintCommand* prt = new PrintCommand(cmd, this, enabled);
    m_print_commands.push_back(prt);

    m_sub_menu->add_toggle(title, enabled  
	,new SimpleCommand<PrintCommand>(prt, &PrintCommand::enable)
	,new SimpleCommand<PrintCommand>(prt, &PrintCommand::disable) );
};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void PrintControl::enable(){    m_enabled = true;}
void PrintControl::disable(){    m_enabled = false;}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void 
PrintControl::printAll()
{
    if(m_enabled) printNow();
}
void
PrintControl::printNow()
{
    for( PrintList::iterator it = m_print_commands.begin(); it !=  m_print_commands.end(); ++it) 
	(*it)->execute();
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void
PrintControl::finishSetup()
{

    m_sub_menu->add_separator();
    m_sub_menu->add_toggle("print after event", true, 
	new SimpleCommand<PrintControl>(this,&PrintControl::enable),
	new SimpleCommand<PrintControl>(this,&PrintControl::disable));
    m_sub_menu->add_button("print now", new SimpleCommand<PrintControl>(this,&PrintControl::printNow) );

    // finally add the messenger toggle 
    m_sub_menu->add_toggle("Messages", true,
	new SimpleCommand<Messenger>
	      (&messenger,
	       &Messenger::enable),
	       new SimpleCommand<Messenger>
	           (&messenger, &Messenger::disable));
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void PrintControl::clear(){m_print->clear();}

void PrintControl::flush(){m_print->flush();}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//   stuff for singleton (deprecated)
PrintControl* PrintControl::s_instance = 0;

PrintControl* PrintControl::instance()
{
    assert( s_instance)	;
    return s_instance;
}

