// Project	: General Utilities Library
// Subproject	: Log
// File		: Warning.h
// Author	: Patrick Hendriks
// Version	: 1.0
// Created	: 02-07-1997 (1.0)
// Last revision: 21-04-1998 (1.0)
//
// Description	:
//	Warning is a messager with a prefix of "WARNING: ". Also, a warning is
//	always printed on a new line. For the rest, the implementation in
//	Messenger is used.
//
// Interface	:
//	Warning()
//	    Create a new warning object.
//
//	virtual std::ostream& stream()
//	    If ms_new_warning is set, advance to the next line before calling
//	    the same method in Messenger.
//
//	virtual Messenger& operator<<(std::ostream& (*p_func)(std::ostream&))
//	    Set ms_new_warning to true and call the corresponding method in
//	    Messenger.
//
// Global variables :
//	Warning warning
//	    Common warning object to be used by all clients.


#ifndef UTIL_LOG_WARNING_H
#define UTIL_LOG_WARNING_H

#include "Messenger.h"

namespace util {


class Warning : public Messenger
{
public:
    Warning();
    // Implicit destructor.
    // Implicit copy constructor and assignment operator.

    std::ostream&			stream();
    virtual Messenger&			operator<<(std::ostream& (*p_func)(std::ostream&));
protected:
    static bool				ms_new_warning;
};


// *************************************************************************
// Global variables:

extern Warning warning;


}	// End of the util namespace

#endif	// UTIL_LOG_WARNING_H
