// $Id: Plane.cxx,v 1.1.1.1 2000/04/03 21:03:53 wilko Exp $
//

#ifdef __GNUG__
#pragma implementation
#endif

#include "Plane.h"
#include <float.h>

Plane::Plane(const Point& o, const Vector& n)
: Surface(o, n.unit())	 // save unit vector in Surface
, m_d(n.magnitude())		 // magnitude here
{ 
   if(m_d==0)
   FATAL("Plane constructor called with null unit vector");

}
Plane::Plane(const Point& o, const Vector& n, double dist)
: Surface(o, n.unit())	 // save unit vector in Surface
, m_d(dist)
{}


double 
Plane::distance( const Point& x, const Vector& vhat, int inout)const
{
  // distance along direction v from point x to enter or leave the surface.
   double denom = vhat*direction();
   return ( denom*inout<=0 )? FLT_MAX : how_near(x)/denom;
}

void
Plane::reverse()
{   
    Surface::reverse();
    m_d *=-1.;


}
Surface&
Plane::copy()const
{
    return *new Plane(origin(), direction(), m_d);
}
void
Plane::printOn( std::ostream& os ) const
{ 	os << "Plane: origin= "	<< origin() 
       << ",\t normal= " << direction() 
	   << ",\t distance= " << m_d
       << "\n";
}





