#ifndef __BEAMTAGGER_H
#define __BEAMTAGGER_H 1

// #include <stdlib.h>
#include <iostream>
#include <vector>
#include "Event/trsDataVI.h"

//----------------------------------------------
//
//   BeamTagger
//
//   Transient Storage Clases
//----------------------------------------------
//   It contains the Raw tagger data
//----------------------------------------------
//             M. Hirayama, Santa Cruz 03/15/00
//----------------------------------------------

//##############################################
class BeamTagger : public trsDataVI
//##############################################
{
public:

	// constructor
	BeamTagger()  {ini();}
	~BeamTagger() {clear();}
	
	// data operations
	virtual void clear();
	virtual void make() {}
	void writeOut() const;

	// set values
	void setClean(bool clean)   {m_clean=clean;}
	void setXBeam(double xbeam) {m_xbeam=xbeam;}
	void setYBeam(double ybeam) {m_ybeam=ybeam;}
	
	inline void setGoodG(bool goodGamma)  { m_goodGamma = goodGamma;}
	inline void setGoodE(bool goodElectron)  { m_goodElectron = goodElectron;} 
	inline void setGoodK(bool goodKion)  { m_goodKion = goodKion;} 
	inline void setGoodH(bool goodHadron)  { m_goodHadron = goodHadron;}
	inline void setGoodPi(bool goodPion) { m_goodPion = goodPion;} 
	inline void setGoodP(bool goodProton)  { m_goodProton = goodProton;} 

	void addParticle(int pid, double mom) {m_PIDList.push_back(pid);m_momList.push_back(mom);}

	// access
	inline bool clean() const {return m_clean;}
	inline int XBeam() const {return m_xbeam;} 
	inline int YBeam() const {return m_ybeam;} 

	inline bool getGoodG() const {return m_goodGamma;}
	inline bool getGoodE() const {return m_goodElectron;} 
	inline bool getGoodK() const {return m_goodKion;} 
	inline bool getGoodH() const {return m_goodHadron;}
	inline bool getGoodPi() const {return m_goodPion;} 
	inline bool getGoodP() const {return m_goodProton;} 
	// access list
	inline int    nParticle()       {return m_PIDList.size();}
	inline int    particleID(int i) {return m_PIDList[i];}
	inline double momentum(int i)   {return m_momList[i];}

private:

	virtual void ini();

private:

	bool m_clean;                  // 1 if clean beam (no halo), else 0
	double m_xbeam;                // x of primary beam in mm
	double m_ybeam;                // y of primary beam in mm
	std::vector<int>    m_PIDList; // 1=e, 2=gamma, 3=proton, 4=pion, 5=kaon
	std::vector<double> m_momList; // momentum each particle (GeV/c)

	bool m_goodGamma;
	bool m_goodElectron;
	bool m_goodKion;
	bool m_goodHadron;
	bool m_goodPion;
	bool m_goodProton;
};
#endif
