#ifndef sirCluster_h
#define sirCluster_h

#include "reconstruction/sirID.h"
#include "geometry/Point.h"

//! SiRecon class to store the a silicon cluster
/*!
	It uses sirID as locate the cluster in the Silicion tracker 
	It has a flag value to be used by the reconstruction algorithms
*/
//##############################
class sirCluster : public sirID
//##############################
{
public:

	friend class sirClusterList;

public:

	//! default constructor
	sirCluster(){clear();}
	//! constructor, set the ID, view, layer, strip initial and final and ToT
	sirCluster(int id, int v, int ilayer, 
		int istrip0, int istripf, double ToT);
	//! default destructor
	~sirCluster() {}

	//! set the Point position of the cluster
	inline void setPosition(Point p)    {m_position = p;}
	//! set the ID of the cluster
	inline void setID(int id)    {m_id = id;}
	//! set the flag value - used by the reconstruction algorithms
	inline void flag(int flag=1) {m_flag = flag;}
	//! unflag the cluster - used by the reconstruction algorithms
	inline void unflag()         {m_flag = 0;}
	
	//returns the id of the cluster
	inline int id() const {return m_id;}
	//! returns the chip ID
	inline int chip()      const {return m_chip;}
	//! returns the central strip
	/*! It is a double beacause the clusters can have several strips */
	inline double strip()  const {return m_strip;}
	//! for compatibility: used intead \c flagged();
	bool Hitflagged()      const {return flagged();}
	//! returns true if the cluster is flagged
	bool flagged()      const {return (m_flag!=0);}
	//! returns the Point position of the cluster
	Point position()       const {return m_position;}
	//! returns the size of the clusters (number of strips)
	inline double size()   const {return m_size;}

	// writeOut some information of the cluster
	void writeOut() const;

protected:

	//! reset all the values
	void clear();

private:

	int m_chip;

	int m_strip0;
	int m_stripf;
	double m_strip;

	double m_size;
	double m_ToT;
	Point m_position;

	int m_id;
	int m_flag;
};
#endif