#ifndef sirClusterList_h
#define sirClusterList_h

#include "Event/trsDataVI.h"
#include "reconstruction/sirBase.h"
#include "reconstruction/sirCluster.h"

#include <vector>

//! transient data to store the Silicon Clusters


//###################################################
class sirClusterList : public trsDataVI
//###################################################
{
public:

	//! constructor
	sirClusterList()    {ini();}
	//! destructor
	~sirClusterList()   {clear();}
	
	//! add a sirCluster
	void addCluster(sirCluster* cl);

	//! returns number of sirCluster s
	int numClusters()  const {return m_clustersList.size();}
	//! returns sirCluster in position i (i is used as the ID of the sirCluster)
	sirCluster* getCluster(int i) const {return m_clustersList[i];}

	//! flag a given cluster 
	void flag(int id, int iflag=1)   {getCluster(id)->flag(iflag);}
	//! unflag a given cluster
	void unflag(int id) {getCluster(id)->unflag();}
	//! returns tree if the cluster is flagged
    bool flagged(int id) const {return getCluster(id)->flagged();}
	//! returns position of the a given Cluster
	Point const position(int id)   {return getCluster(id)->position();} 
	//! returns the size of a given cluster
	double const size(int id)      {return getCluster(id)->size();}     

	//! returns a list of sirCluster
	/*! returns a reference to a vector<sirCluster*> list of clusters
	of a given view an plane */
	const std::vector<sirCluster*>& getClusters(sirBase::VIEW v, int iplane) const
	{return m_clustersByPlaneList[sirBase::viewToInt(v)][iplane];}
	//! returns the number of cluster in a given plane and view
	int numClusters(sirBase::VIEW v, int iplane) const 
	{return m_clustersByPlaneList[sirBase::viewToInt(v)][iplane].size();}

	//! flagg al Clusters in plane with view v and int iplane
	void flagClustersInPlane(sirBase::VIEW v, int iplane);

	//! writeOut the information of the clusters
	void writeOut() const;
//	void update(ArvePlottableRep& v)  {draw(v);}

	//! clear the storage - delete the sirClusters
	virtual void clear();
	//! null the \c make() method
	virtual void make() {}

protected:

	//! returns a point with the average of all clusters in a plane and view 
	Point meanPoint(sirBase::VIEW v, int iplane);
	//! returns a point with the average of all clusters in
	/*! a radius size around the a given point Pini*/
	Point meanPointInside(sirBase::VIEW v, int iplane, double size, Point Pini);
	//! returns a point an a sirCluster Id (via pointer) of
	/*! the nearest SirCluster of a given plane and view, outside a radius
	in radius centered around a given point centerX*/
	Point nearestPointOutside(sirBase::VIEW v, int iplane, double inRadius, 
		Point centerX, int& id);

private:

	//! initialize the vectors of sirCluster
	virtual void ini();

//	//! plots the sirClusters
//	void draw(ArvePlottableRep& v);

private:

	//! List of all the sirCluster s
	std::vector<sirCluster*> m_clustersList;
	//! List of all sirClusters organized by plane and views
	/*! the \c addCluster() method handle how to add a cluster in both
	containers <br> the clusters are organized in two lists:<ul>
	    <li> One containes the list of all clusters
	    <li >the other one has them ordered by plane and view to
	    facilitate access to Pattern Recognition functions </ul>*/
	std::vector<sirCluster*> 
		m_clustersByPlaneList[NVIEWS][NPLANES]; 
};

#endif