#ifndef sirError_h
#define sirError_h

//! protected class to store the errors of the covariance matrix
//###############################
class sirError
//###############################
{
public:

	friend class sirLocator;

protected:

	//! default constructor
	sirError(){clear();}
	//! constructor set the errors.
	/*! Note that the errors are the sigmas and not the sigmas squared */
	sirError(double sX, double sS, double cov):m_sigmaX(sX),m_sigmaSlope(sS),m_cov(cov) {}
	//! default destructor
	~sirError() {}

	//! returns error in the position
	inline double locate() const {return m_sigmaX;}
	//! returns error in the slope
	inline double slope()    const {return m_sigmaSlope;}
	//! returns covariance element between position and slope
	inline double cov()      const {return m_cov;}
	//! returns error square of the position
	inline double locateSq() const {return m_sigmaX*m_sigmaX;}
	//! returns error square of the slope
	inline double slopeSq()  const {return m_sigmaSlope*m_sigmaSlope;}

	void clear() {m_sigmaX=0.;m_sigmaSlope=0.;m_cov=0.;}
private:

	//! error in the position
	double m_sigmaX;
	//! error in the slope
	double m_sigmaSlope;
	//! covariance element between position and slope
	double m_cov;
};
#endif