#ifndef sirLocator_h
#define sirLocator_h

#include "reconstruction/sirError.h"
#include "geometry/Point.h"

//! class to locate a particle in space
/*!
	This class does not know anything about the Silicon tracker
*/
//###############################
class sirLocator
//###############################
{
public:

	//! constructor - clear data
	sirLocator() {clear();}
	//! defautl destructor
	~sirLocator() {}

	//! set energy
	inline void setEnergy(double ene)        {m_energy = ene;}
	//! set error in energy
	inline void setSigmaEnergy(double sigma) {m_sigmaEnergy = sigma;}
	//! set position with x, y and z
	void setPosition(double x, double y, double z) {m_position = Point(x,y,z);}
	//! set the position with a Point
	void setPosition(Point p)                      {m_position = p;}
	//! set the direction with x,y,z
	void setDirection(double x, double y, double z) {m_direction = Vector(x,y,z);}
	//! set the direction with a vector
	void setDirection(Vector v)                    {m_direction = v;}
	//! set X covariance matrix
	void setXcov(double sX, double sS, double cov) {m_xCov = sirError(sX,sS,cov);}
	//! set Y covariance matrix
	void setYcov(double sX, double sS, double cov) {m_yCov = sirError(sX,sS,cov);}
	
	//! returns energy
	inline double energy()   const {return m_energy;}
	//! returns error in energy
	inline double sigmaEnergy() const {return m_sigmaEnergy;}
	//! returns a Point with the position 
	inline Point position() const {return m_position;}
	//! returns a Vector with the direction
	inline Vector direction() const {return m_direction;}

	//! returns the x position
	double X() const {return position().x();}
	//! returns the y position
	double Y() const {return position().y();}
	//! returns the z position
	double Z() const {return position().z();}
	//! returns the X slope
	double slopeX() const;
	//! returns the Y slope
	double slopeY() const;
	//! returns the X position error
	double sigmaX() const {return covX().locate();}
	//! returns the X slope error
	double sigmaXslope() const {return covX().slope();}
	//! returns the X position-slope convariance element
	double sigmaXcov() const {return covX().cov();}
	//! returns the Y position error
	double sigmaY() const {return covY().locate();}
	//! returns the Y slope error
	double sigmaYslope() const {return covY().slope();}
	//! returns the Y position-slope convarinace element
	double sigmaYcov() const {return covY().cov();}

protected:

	void clear();

	//! returns sirError of the X projection of covariance matrix
	const sirError& covX() const {return m_xCov;}
	//! returns sirError of the Y projection of covariance matrix
	const sirError& covY() const {return m_yCov;}

private:

	//! energy
	double m_energy;
	//! error in the energy
	double m_sigmaEnergy;
	//! Point position
	Point m_position;
	//! vector with the direction
	Vector m_direction;
	//! sirError X covariance matrix
	sirError m_xCov;
	//! sirError Y covariance matrix
	sirError m_yCov;
};
#endif