#ifndef sirQuality_h
#define sirQuality_h

//! base class that stores the fitting quality of a SIRec object
/*!
  There are some quality variables that define the quality fitting.
  We are using: chiSquare, a Quality and residuals
*/

//#############################
class sirQuality
//#############################
{
public:

	//! constructor - clear the data
	sirQuality() {clear();}
	//! defualt destructor
	~sirQuality() {}

	//! set the chi2
	inline void setChiSq(double chi)    {m_chiSq = chi;}
	//! set the quality
	inline void setQuality(double q)    {m_quality = q;}
	//! set the residuals
	inline void setResidual(double res) {m_residual = res;}

	//! returns chiSquare
	inline double chiSq()    const {return m_chiSq;}
	//! returns quality
	inline double quality()  const {return m_quality;}
	//! returns residuals
	inline double residual() const {return m_residual;}

protected:

	virtual void clear() {m_chiSq=0.;m_quality=0.;m_residual=0.;}

private:

	//! chiSquare value
	double m_chiSq;
	//! quality value
	double m_quality;
	//! residual value
	double m_residual;

};
#endif