#ifndef sirTrackList_h
#define sirTrackList_h

#include "Event/trsDataVI.h"

#include "RootTree/TkrRecon.h"
#include "RootTree/TkrSiCluster.h"
#include "RootTree/TkrHit.h"
#include "RootTree/TkrLocator.h"
#include "RootTree/TkrTrack.h"
#include "RootTree/TkrGamma.h"

#include "reconstruction/sirBase.h"
#include "reconstruction/sirTrack.h"
#include "reconstruction/sirGamma.h"

//------------------------
//! server of the silicon tracks - transient data
/*!
\e sirTrackList stores, servers the reconstructed tracks.<br>
\e sirTrackList is a transient data storage
*/
//-------------------------
// JA Hernando, Santa Cruz, Ca, 06/16/00
//-------------------------
//#########################
class sirTrackList : public trsDataVI
//#########################
{
public:

	//! default constructor
	sirTrackList() {ini();}
	//! default destructor
	~sirTrackList() {clear();}

	//! number of tracks
	int numTracks() const {return m_list.size();}
	//! add silicon track into the list
	void addTrack(sirTrack* tr);
	//! get silicon track from the list
	sirTrack* getTrack(int i) {return (i >=0 && i < numTracks()? m_list[i]:0);}

	//! number of X tracks
	int numXTracks() const {return numTracks(sirBase::X);}
	//! get X track
	sirTrack* getXTrack(int i) {return (i>=0 && i <numXTracks()? getTrack(sirBase::X,i) : 0);}

	//! number of Y tracks
	int numYTracks() const {return numTracks(sirBase::Y);}
	//! get X track
	sirTrack* getYTrack(int i) {return (i>=0 && i < numYTracks()?getTrack(sirBase::Y,i) : 0);}

	//! add sirGamma
	void addGamma(sirGamma* gamma) {m_gamma = gamma;}
	//! get sirGamma
	sirGamma* gamma() const {return m_gamma;}

	//! clear the list of tracks - delete them
	virtual void clear();
	//! null make function
	virtual void make() {};

protected:

	//! X and Y Tracks
	int numTracks(sirBase::VIEW v) const {return m_listByView[sirBase::viewToInt(v)].size();}
	//! X and Y Tracks
	sirTrack* getTrack(sirBase::VIEW v, int i) {return (m_listByView[sirBase::viewToInt(v)])[i];}

private:

	//! ini - clear the list
	virtual void ini() {m_list.clear();m_gamma=0;};

protected:

	//! list with the pointer to the silicon tracks
	std::vector<sirTrack*> m_list;
	//! list with the pointer to the silicon tracks organized by view
	std::vector<sirTrack*> m_listByView[NVIEWS];

	sirGamma* m_gamma;
};
#endif