#include "rTreeRawServer.h"
#include "ACDTile.h"
#include "CalHit.h"
#include "StripID.h"
#include "TkrLayer.h"
#include "Tagger.h"


#include "TTree.h"

#include "RootTree/converterImport.h"

//#######################################
void rTreeRawServer::defineConverters()
//#######################################
{
	/*! add the converter to SiLayers */
	addConverter("SiLayers",  
		new converterImport<rTreeRawServer,SiLayers>(this,"SiLayers") );
	
	/*! add the converter from CalRecLogs */
	addConverter("CalRawLogs", 
		new converterImport<rTreeRawServer,CalADCLogs>(this,"CalRawLogs") );
	
	/*! add the converter from tbHeader */
	addConverter("BeamTagger",	 
		new converterImport<rTreeRawServer,BeamTagger>(this,"BeamTagger") );

	/*! add the converter from rawEventID */
	addConverter("rawEventID",	 
		new converterImport<rTreeRawServer,rawEventID>(this,"rawEventID") );
	
	/*! create the load and fill algorithms with the converters */
	makeAlgorithms();
}
//--------------------------------------------------------------

//###############################################
void rTreeRawServer::load(rawEventID* rawEvt)
//###############################################
{
	int evt = core()->getEvent();
	rawEvt->setEventID(evt);
	int run = core()->getRun();
	rawEvt->setRunID(run);
}
//###############################################
void rTreeRawServer::load(SiLayers* siLayers)
//###############################################
{
	int OFFSET = 64;   // threre is an offser in the idaddress
	int nTKR = core()->getTKR()->GetEntries();
	for (int i = 0; i < nTKR; i++) {
		
		TkrLayer* rLayer = (TkrLayer*) core()->getTKR()->At(i);
		int XY = (rLayer->getXY() == TkrLayer::X ? 1 : 0);
		double tot = rLayer->getTOT(0);
		//double tot = 0;
		
		int nStrips = rLayer->getStrips()->GetEntries();
		if (nStrips >0 ) {
			SiLayer* siLayer = new SiLayer(rLayer->getLayer(), XY, tot);
			StripID* rStrip;
			for (int iStrip=0; iStrip < nStrips; iStrip++) {
				rStrip = (StripID*) rLayer->getStrips()->At(iStrip);
				int iaddress = rStrip->getID() - OFFSET;
				siLayer->addStrip((int) iaddress);
			}
			siLayers->add(siLayer);
		}
	}
}
//###############################################
void rTreeRawServer::load(CalADCLogs* calLogs)
//###############################################
{	
	int iread = 0;
	double BASE = 4095; // the 0 for the ADC's
	int NLAYERS = 3;
	int nCAL = core()->getCAL()->GetEntries();
	for (int i = 0; i < nCAL; i++) {
		CalHit* rLog = (CalHit*) core()->getCAL()->At(i);

		LogID*  rLogID= rLog->getLogID();

		int ADCID   = rLogID->getID();
		int iplane = ADCID/2;
		iplane = NLAYERS -  (iplane%4);
		int icolumn = ADCID/8;
		int iview   = 1-ADCID%2;  
		// even are Y-layers 
		int jplane  = rLogID->getLayer()/2;
		jplane      = NLAYERS-jplane;
		int jcolumn  = rLogID->getColumn();
		int jview   = 1-rLogID->getXY();
		// JA new convention 04/15/00
		//jview = 1-jview;
		if (iview == 0) icolumn = 9 - icolumn; 

		CalADCLog* log = calLogs->getLogID(CalLogID::ID(iplane,detGeo::makeAxis(iview),icolumn));

		log->setNegADC(CalBase::LEX,BASE-rLog->getADCValue(CalHit::Neg,CalHit::LowEnX4));
		log->setNegADC(CalBase::LE, BASE-rLog->getADCValue(CalHit::Neg,CalHit::LowEn));
		log->setNegADC(CalBase::HEX,BASE-rLog->getADCValue(CalHit::Neg,CalHit::HiEnX8));
		log->setNegADC(CalBase::HE, BASE-rLog->getADCValue(CalHit::Neg,CalHit::HiEn));

		log->setPosADC(CalBase::LEX,BASE-rLog->getADCValue(CalHit::Pos,CalHit::LowEnX4));
		log->setPosADC(CalBase::LE, BASE-rLog->getADCValue(CalHit::Pos,CalHit::LowEn));
		log->setPosADC(CalBase::HEX,BASE-rLog->getADCValue(CalHit::Pos,CalHit::HiEnX8));
		log->setPosADC(CalBase::HE, BASE-rLog->getADCValue(CalHit::Pos,CalHit::HiEn));
	}
}
//###############################################
void rTreeRawServer::load(BeamTagger* tagger)
//###############################################
{	
	ESAPID* pid = core()->getTagger()->getPID();

	// beam info
	tagger->setClean(pid->getClean());
	tagger->setXBeam(pid->getXbeam());
	tagger->setYBeam(pid->getYbeam());
	//Quality info
	tagger->setGoodG(pid->getOneGoodG());
	tagger->setGoodE(pid->getOneGoodE());	
	tagger->setGoodH(pid->getOneGoodH());	
	tagger->setGoodP(pid->getOneGoodP());
	tagger->setGoodPi(pid->getOneGoodPi());	
	tagger->setGoodK(pid->getOneGoodK());
	// particle info
	int nPart = pid->getNpart();
	for (int i = 0; i < nPart; i++) {
		tagger->addParticle((int)pid->getIDN(i), (double)pid->getMomN(i));
	}
}
