#ifndef rTreeRawServer_h
#define rTreeRawServer_h

#include "Event.h"

#include "Event/rTreeBase.h"
#include "Event/converterServer.h"

#include "generation/SiLayers.h"
#include "instrument/CalADCLogs.h"
#include "generation/BeamTagger.h"
#include "generation/rawEventID.h"

//############################################
class rTreeRawServer : public rTreeBase<Event>, public converterServer
//#############################################
{	
public:

	//! constructor: create the converters
	/*! the constructor of converterServer will call defineConverters*/
	rTreeRawServer():rTreeBase<Event>(new Event(),"Event") {defineConverters();}
	//! default destructor
	~rTreeRawServer(){};

	//! load Raw data into rawEventID
	/*! the run and event ID */
	void load(rawEventID* rawEvt); 
	//! load Raw data into "SiLayers" transient data class
	/*! data with the information of the Si-tracks: strips, ToT, etc*/
	void load(SiLayers*   siLayers); 
	//! export Rae data into "CalRawLogs" transiend data class
	/*! data with the ADC counts of the different range/ends of the calorimter logs */
	void load(CalADCLogs* calLogs); 
	//! export Raw data into "BeamTagger" transient data class
	/*! data with the beam information provided by the tagger */
	void load(BeamTagger* tagger); 

	//! null save rawEventID
	void save(rawEventID* rawEvt) {}; 
	//! null save SiLayers
	void save(SiLayers*   siLayers) {}; 
	//! export Rae data into "CalRawLogs" transiend data class
	void save(CalADCLogs* calLogs) {};
	//! export Raw data into "BeamTagger" transient data class
	void save(BeamTagger* tagger) {}; 

protected:

	//! The User should define in this methods the converters from Recon->Transient
	/*! The system will not know the persistent data and will interact only
	with the algorithms created for the converters that the User declare here */
	virtual void defineConverters();
};
#endif

