#ifndef sirGamma_h
#define sirGamma_h

#include "reconstruction/sirQuality.h"
#include "reconstruction/sirLocatorID.h"

#include "reconstruction/sirTrack.h"

//-------------------------
//! the tracker reconstructed gamma
//-------------------------
// JA Hernando, Santa Cruz, 06/16/00
//-------------------------
//#########################
class sirGamma : public sirQuality, public sirLocatorID
//#########################
{
public:

	//! default constructor 
	sirGamma() {m_tracks.clear();}
	//! default destructor - does not destroies the tracks
	~sirGamma() {clear();}

	//! returns number of tracks
	int numTracks() const {return m_tracks.size();}
	//! adds track to the list
	void addTrack(sirTrack* track) {m_tracks.push_back(track);}
	//! returns a track in position i
	sirTrack* getTrack(int i) const {return (i>=0 && i < numTracks()?m_tracks[i]:0);}

	//! returns the number of X tracks of the gamma
	int numXTracks() const {return numTracks(sirBase::X);};
	//! returns the number of Y tracks of the gamma 
	int numYTracks() const {return numTracks(sirBase::Y);};

	//! returns the tracke in position i with X or Y projextion
	sirTrack* getTrack(sirBase::VIEW v, int itrack) const;

protected:

	int numTracks(sirBase::VIEW v) const;

	//! clear the list
	void clear() {m_tracks.clear();}

private:

	//! list with the pointers to the track of this gamma
	std::vector<sirTrack*> m_tracks;
};
#endif