#ifndef sirID_h
#define sirID_h

#include "reconstruction/sirBase.h"

//! base class to define an id for SIReconstruction classes

//##############################
class sirID
//##############################
{
public:

	//! default constructor
	sirID() {clear();}
	//! default destructor
	~sirID() {}

	//! set id
	inline setId(int id)         {m_id = id;}
	//! set tower
	inline setTower(int itower)  {m_tower = itower;}
	//! set layer
	inline setPlane(int plane)   {m_plane = plane;}
	//! set the view with an integer
	inline setView(int iview)     {m_view = sirBase::intToView(iview);}
	//! set the view with an enum VIEW
	inline setView(sirBase::VIEW view)   {m_view = view;}
	//! set the energy
	inline setEnergy(double ene) {if (ene >= sirBase::minEnergy()) m_energy = ene;}

	//! returns the object id
	inline int id() const {return m_id;}
	//! returns the tower ID
	inline int tower() const {return m_tower;}
	//! returns the plane ID
	inline int plane() const {return m_plane;}
	//! returns the enum VIEW
	inline sirBase::VIEW getView() const {return m_view;}
	//! returns the energy
	inline double energy() const {return m_energy;}

	//! clear the ID
	void clear() {
		m_id = -1; m_tower = -1; m_plane=-1; 
		m_view = sirBase::XY; m_energy = sirBase::minEnergy();
	}

private:

	//! object id
	int m_id;
	//! tower ID
	int m_tower;
	//! silicon plane ID 
	int m_plane;
	//! View
	enum sirBase::VIEW m_view;
	//! energy
	double m_energy;
};
#endif