#ifndef sirLocatorID_h
#define sirLocatorID_h

#include "reconstruction/sirID.h"
#include "reconstruction/sirLocator.h"

//! It is a sirLocator with a sirID.
/*! 
	This class is a sirLocator that knows about the Silicion thanks to the
	sirID.<br>
	Thanks to sirID, we can ask for the \c slope(), \c location() \c sigmaLocation(),
	etc without been worry if it is X or Y projection. If it is 3D nothing is returned.<br>
	It makes the distition of input energy (energy in sirID) and reconstructed
	energy (energy in sirLocator). The \c energy() method returns the reconstructed
	energy.
*/

//##################################
class sirLocatorID: public sirID, public sirLocator
//##################################
{
public:

	//! default constructor
	sirLocatorID() {}
	//! default destructor
	~sirLocatorID() {}

	//! set the input energy (energy in sirID);
	void setInputEnergy(double ene) {sirID::setEnergy(ene);}
	//! set the reconstructed energy (energy in sirLocator)
	void setRecEnergy(double ene)  {sirLocator::setEnergy(ene);}
	//! resolve ambiguities, set the energy as reconstructed energy
	void setEnergy(double ene) {sirLocator::setEnergy(ene);}

	//! returns the location
	double location() const;
	//! returns the slope;
	double slope() const;
	//! returns the error in the location
	double sigmaLocation() const;
	//! returns the error in the slope
	double sigmaSlope() const;
	//! returns the location - slope convariance element
	double sigmaCov() const;

	//! returns the input energy (energy in sirID)
	double inputEnergy() const {return sirID::energy();}
	//! returns the reconstructed energy (energy in sirLocator)
	double recEnergy() const {return sirLocator::energy();}
	//! solve ambiguities, returns the energy in sirLocator
	double energy() const {return sirLocator::energy();}

};
#endif