#include "reconstruction/sirCluster.h"
#include "Event/messageManager.h"

//###################################################
sirCluster::sirCluster(int id, int v, int ilayer, 
					 int istrip0, int istripf, double ToT)
//###################################################
{
	clear();

	/*! set sirID of the cluster*/
	setId(id);
	setView(v);
	/*! planes are ordered from 0-top to 15-bottom   - used by the recostruction <br>
	layers are ordered from 15-top to 0-bottom   - used by the geometry */
	setPlane(sirBase::numPlanes()-1 - ilayer);

	m_strip0 = istrip0;
	m_stripf = istripf;
	m_chip = (int) m_strip0/64;

	m_strip = 0.5*(m_strip0+m_stripf);
	m_size = fabs(m_stripf-m_strip0+1);
	
	m_ToT = ToT;
}

//###################################################
void sirCluster::clear()
//###################################################
{	
	sirID::clear();
	m_chip   = -1;
	m_flag   = 0;
	m_id     = -1;
	m_position = Point(0.,0.,0.);
	m_size   = 0;
	m_strip  = 0;
	m_strip0 = 0;
	m_stripf = 0;
	m_ToT    = 0.;
}

//######################################################
void sirCluster::writeOut() const
//######################################################
{
	
	std::ostream& out = messageManager::instance()->out();
	if (!messageManager::instance()->acceptLevel("DEBUG")) return;

//	out << "ID     " << m_id    << " Flag   " << m_flag  << "\n";
//	out << "tower  " << m_tower << " plane  " << m_plane << " view  " << m_view << " chip " << m_chip << "\n";
//	out << "strip0 " << m_strip0<< " stripf " << m_stripf<< " strip " << m_strip<< " size " << m_size << "\n"; 

	out << " plane " << plane() << " XY " << getView();
	out << " xpos   " << m_position.x()  << " ypos   " << m_position.y();
	out << " i0-if " << m_strip0 <<"-"<< m_stripf;
	out <<"\n";
}