#include "reconstruction/sirClusterList.h"

#include "Event/messageManager.h"

//######################################################
void sirClusterList::addCluster(sirCluster* cl)
//######################################################
{
	m_clustersList.push_back(cl);
	int iview = sirBase::viewToInt(cl->getView());
	m_clustersByPlaneList[iview][cl->plane()].push_back(cl);
}
//###################################################
void sirClusterList::clear()
//###################################################
{
	int nClusters = m_clustersList.size();
	for (int iCluster = 0; iCluster < nClusters; iCluster++) {
		delete m_clustersList[iCluster];
	}
	ini();
}
//###################################################
void sirClusterList::ini()
//###################################################
{
	m_clustersList.clear();
	for (int iview = 0; iview < NVIEWS; iview++) {
		for (int iplane = 0; iplane < NPLANES; iplane++) {
			m_clustersByPlaneList[iview][iplane].clear();
		}
	}
}
//######################################################
Point sirClusterList::meanPoint(sirBase::VIEW v, int iplane)
//######################################################
{
	Point Pini(0.,0.,0);

	int nClusters = numClusters(v,iplane);
	if (nClusters == 0) return Pini;

	std::vector<sirCluster*> AuxList = getClusters(v,iplane);
	for (int iCluster=0; iCluster<nClusters; iCluster++){
		Pini += AuxList[iCluster]->position();	
	}
	Point Pini2(Pini.x()/nClusters,Pini.y()/nClusters,Pini.z()/nClusters);
	return Pini2;
}

//######################################################
Point sirClusterList::meanPointInside(sirBase::VIEW v, int iplane, double radius,
								Point Pcenter)
//######################################################
{
	Point P(0.,0.,0);
	std::vector<sirCluster*> AuxList = getClusters(v,iplane);
	int nClusters = AuxList.size();
	if (nClusters == 0) return P;

	int nsum =0;
	double xsum = 0.;
	double ysum = 0.;
	double zsum = 0.;
	for (int iCluster=0; iCluster<nClusters; iCluster++){
		P = AuxList[iCluster]->position();
		if ((P-Pcenter).mag() < radius) {
			nsum++;
			xsum+=P.x();
			ysum+=P.y();
			zsum+=P.z();
		}
	}
	Point Pini(xsum/(1.*nsum),ysum/(1.*nsum),zsum/(1.*nsum));
	return Pini;
}

//####################################################################
Point sirClusterList::nearestPointOutside(sirBase::VIEW v, int iplane, 
								 double inRadius, Point Pcenter, int& id)
//####################################################################
{
	Point Pnear(0.,0.,0.);
	id = -1;

	int nClusters = numClusters(v,iplane);
	if (nClusters == 0) return Pnear;

	std::vector<sirCluster*> AuxList;
	AuxList = getClusters(v,iplane);

	double minRadius = inRadius;
	double maxRadius = 1e6;
	Point Pini(0.,0.,0.);
	for (int iCluster = 0; iCluster< nClusters; iCluster++) {
		Pini = AuxList[iCluster]->position();
		double radius; 
		if (v == sirBase::X) radius = fabs(Pini.x()-Pcenter.x());
		else if (v == sirBase::Y) radius = fabs(Pini.y()-Pcenter.y());
		else radius = (Pini-Pcenter).mag();
		if ( radius > minRadius && radius < maxRadius ) {
			maxRadius = radius;
			Pnear = Pini;
			id = AuxList[iCluster]->id();
		}
	}
	return Pnear;
}
//######################################################
void sirClusterList::flagClustersInPlane(sirBase::VIEW v, int iplane)
//######################################################
{
	std::vector<sirCluster*> AuxList = getClusters(v,iplane);
	for (int iCluster = 0; iCluster< AuxList.size(); iCluster++)
		AuxList[iCluster]->flag();
}

//######################################################
void sirClusterList::writeOut() const
//######################################################
{
	if (numClusters()<=0) return;

	std::ostream& out = messageManager::instance()->out();
	if (!messageManager::instance()->acceptLevel("DEBUG")) return;
	out << "sirCluster in List " << m_clustersList.size() << "\n";

	for (int iCluster = 0; iCluster < numClusters(); iCluster++) {
		m_clustersList[iCluster]->writeOut();
	}
}

